;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;RAM Page 1 layout

;-------------------------$bf04
;	ISR							70
;-------------------------$bebe
;FREE SPACE!!!
;-------------------------$baea
;	Mirrored enemy sprites		490
;-------------------------$b900
;-------------------------$b84e
;	Sprites						1358
;-------------------------$b300
;-------------------------$b2fe
;	Object tables				510
;-------------------------$b100
;MORE FREE SPACE!!!
;-------------------------$a898
;	Tileset						4440
;-------------------------$9740
;	Buffer area for overflow	16
;-------------------------$9700
;	Screen buffer				2048
;-------------------------$8f00
;	Background screen buffer (w/mask)	3072
;-------------------------$8300
;-------------------------$82fe
;   Current screen          	526
;-------------------------$80f0

.plugin bin

#include "86.inc"
#include "key_equ.inc"
#include "ONgfx.inc"

;MEMORY USAGE
vars	equ	$c920
;level
Level_Start_Addr		equ	vars		;3		;Abs addr of start of level data
Curr_Zone				equ	vars+3		;16		;Zero-terminated string w/name of current zone
Curr_Sector				equ	vars+19		;1		;Number of the current sector
Sub_Init_Pos       		equ vars+20		;2		;Initial position of sub in first screen of level
;sub
Sub_Sprite      		equ vars+22		;2		;Pointer to sprite to use for sub
Sub_Sprite_Save			equ	vars+24		;2		;sub_sprite saved here for use by sub flashing routine
Sub_Pos_X       		equ vars+26		;1
Sub_Pos_Y       		equ vars+27		;1
Sub_Pos					equ	Sub_Pos_X			;Position of sub
Old_Sub_Pos_X   		equ vars+28		;1
Old_Sub_Pos_Y   		equ vars+29		;1
Old_Sub_Pos				equ	Old_Sub_Pos_X		;Previous position of sub
Sub_Dir					equ	vars+30		;1		;Direction of movement of sub - 0=right, 1=left
Last_Move				equ	vars+31		;1		;Direction sub last moved
Sub_Delay_Coeff			equ	vars+32		;2		;Delay length to use for sub movement
Sub_Horiz_Delay_Coeff	equ	vars+34		;2
Sub_Vert_Delay_Coeff	equ	vars+36		;2
Sub_Flash				equ	vars+38		;1		;Sub - 0=normal, 1=flashing, invincible
Sub_Flash_Num_Counter	equ	vars+39		;1		;Counter for number of sub flashes before stopping
Sub_Flash_Counter		equ	vars+40		;1		;Delay counter for sub flash
;ink
Num_Ink_Pellets			equ	vars+41		;1		;Number of remaining ink pellets
Ink_Pos_X       		equ vars+42		;1
Ink_Pos_Y       		equ vars+43		;1
Ink_Pos					equ	Ink_Pos_X			;Position of ink pellet
Old_Ink_Pos_X			equ	vars+44		;1
Old_Ink_Pos_Y			equ	vars+45		;1
Old_Ink_Pos				equ	Old_Ink_Pos_X		;Previous position of ink pellet
Ink_State       		equ vars+46		;1		;Status of current ink pellet - 0=no ink, 1=going right, 2=going left, 3=splat, 4=cloud, 5=erase, 6=erase then cloud
Splat_Counter			equ	vars+47		;1		;Delay counter for ink splat delay
Cloud_Counter			equ	vars+48		;2		;Delay counter for ink cloud
Ink_Move_Counter		equ	vars+50		;1		;Delay counter for ink pellet movement
;enemies
Enemy_1_Move			equ	vars+51		;2		;Pointers to current movements for each enemy
Enemy_2_Move			equ	vars+53		;2
Enemy_3_Move			equ	vars+55		;2
Enemy_4_Move			equ	vars+57		;2
Enemy_5_Move			equ	vars+59		;2
Enemy_1_Pos				equ	vars+61		;2		;Current positions of enemies
Enemy_2_Pos				equ	vars+63		;2
Enemy_3_Pos				equ	vars+65		;2
Enemy_4_Pos				equ	vars+67		;2
Enemy_5_Pos				equ	vars+69		;2
Old_Enemy_1_Pos			equ	vars+71		;2		;Previous positions of enemies
Old_Enemy_2_Pos			equ	vars+73		;2
Old_Enemy_3_Pos			equ	vars+75		;2
Old_Enemy_4_Pos			equ	vars+77		;2
Old_Enemy_5_Pos			equ	vars+79		;2
Enemy_Next_Move_Counter	equ	vars+81		;1		;Delay counter for next enemy movement
Enemy_Pos_Ptr			equ	vars+82		;2		;Temp pointer to Enemy_X_Pos location with position of current enemy
Enemy_Old_Pos_Ptr		equ	vars+84		;2		;Same for Old_Enemy_X_Pos
Enemy_Data_Ptr			equ	vars+86		;2		;Temp pointers to property data for current enemy
Enemy_Data_Ptr2			equ	vars+88		;2
Alt_Enemy_Pos_Ptr		equ	vars+90		;2		;Alternate temp pointers to enemy data
Alt_Enemy_Old_Pos_Ptr	equ	vars+92		;2
Alt_Enemy_Data_Ptr		equ	vars+94		;2
Enemy_Move_Counter		equ	vars+96		;1		;Delay counter for moving enemies
Proj_State				equ	vars+97		;1		;Status of current enemy projectile - 0=none, 1=going right, 2=going left, 3=erase
Proj_Move_Counter		equ	vars+98		;1		;Delay counter for enemy projectile movement
Proj_Sprite				equ	vars+99		;2		;Pointer to sprite used for enemy projectile
Proj_Pos_X				equ	vars+101	;1
Proj_Pos_Y				equ	vars+102	;1
Proj_Pos				equ	Proj_Pos_X			;Position of enemy projectile
Old_Proj_Pos			equ	vars+103	;2		;Previous position of enemy projectile
;dolphin
Dolphin_Pos				equ	vars+105	;2		;Position of dolphin
Old_Dolphin_Pos			equ	vars+107	;2		;Previous position of dolphin
Dolphin_Move			equ	vars+109	;2		;Pointer to current move of dolphin
Dolphin_Move_Counter	equ	vars+111	;1		;Delay counters for dolphin move
Dolphin_Next_Move_Counter	equ	vars+112	;1
Curr_Dolphin_Status		equ	vars+113	;2		;Pointer to dolphin status byte for current screen
Dolphin_Enable			equ	vars+115	;1		;State of dolphin - 0=don't move, 1=move
;objects
Fragment_Anim_Counter	equ	vars+116	;1		;Delay counter for capsule fragment animation
Canister_Anim_Counter	equ	vars+117	;1		;Delay counter for data canister animation
Curr_Fragments			equ	vars+118	;2		;Pointer to capsule fragment table for current screen
Curr_Canisters			equ	vars+120	;2		;Pointer to data canister table for current screen
Num_Objects				equ	vars+122	;1		;Number of objects remaining in level
Object_Anim_Enable		equ	vars+123	;1		;State of objects - 0=don't animate, 1=animate
;misc
End_Game_SP				equ	vars+124	;2		;SP saved here for end of game
Start_Of_Line   		equ vars+126	;1		;Start of line in pixels (for _vnewline)
Redraw					equ	vars+127	;1		;Redraw status of ink/enemies - 0=doesn't need to be redrawn, 1=redraw
New_Screen				equ	vars+128	;1		;Redraw status of background - 0=doesn't need to be redrawn, 1=redraw
New_Level				equ	vars+129	;1		;0=not new level, 1=draw message window w/zone, sector
Draw_Status				equ	vars+130	;1		;Status display - 0=don't draw, 1=draw in lower-left corner
Status_Disp_Counter		equ	vars+131	;1		;Delay counter for displaying status
Oxygen					equ	vars+132	;1		;Amount of oxygen left

;GAME CONTROL
Title_Key_Delay_Coeff	equ 18000		;Delay length for keypresses at title menu
Title_Delay_Coeff		equ	65535		;Delay length for title screen / 3
Loop_Delay_Coeff		equ	2400		;Delay length for main loop
Sub_Horiz_Delay_1		equ	6900	;13000
;Sub_Horiz_Delay_2		equ	1900	;7000
Sub_Horiz_Delay_2		equ	1700
Sub_Vert_Delay_1		equ	6000	;11000
;Sub_Vert_Delay_2		equ	1500	;6000
Sub_Vert_Delay_2		equ	1300
Sub_Horiz_Accel			equ	500		;300
Sub_Vert_Accel			equ	350		;280
Max_Enemy_Num			equ	5			;Max. number of enemies in one screen
Enemy_Move_Num			equ	20			;Max. number of movements for each enemy
Dolphin_Move_Num		equ	20
Max_Fragment_Num		equ	2			;Max. number of capsule pieces in one screen
Max_Canister_Num		equ	2			;Max. number of data canisters in one screen
Ink_Splat_Delay			equ	30			;Length of time that ink splat appears
Ink_Cloud_Delay			equ	200			;Length of time that ink cloud appears
Ink_Move_Delay			equ	3			;Delay between ink pellet moves
Proj_Move_Delay			equ	3			;Delay between enemy projectile moves
Status_Delay			equ	255			;Length of time that status display is shown
Enemy_Move_Delay		equ	20			;Delay between enemy moves
Enemy_Move_Dist			equ	2			;Distance enemy is moved with each cycle
Enemy_Next_Move_Delay	equ	3			;Number of cycles enemy spends on each movement
Dolphin_Move_Delay		equ	25			;(same settings for dolphin)
Dolphin_Move_Dist		equ	2
Dolphin_Next_Move_Delay	equ	4
Oxygen_Start			equ	8			;Amount of oxygen at start of game
Ink_Start				equ	21			;Number of ink pellets at start of game
Sub_Flash_Num			equ	20			;Number of times sub flashes (when hit)
Sub_Flash_Time			equ	25			;Delay between each flash of sub
Fragment_Anim_Delay		equ	180			;Delay for animation of capsule pieces
Canister_Anim_Delay		equ	17			;Delay for animation of data canisters

;OTHER CONSTANTS
Pic_Len                 equ 1024        ;Length of single screen-size pic
Map_Len					equ	16*8		;Length of single screen-size tilemap
Enemy_Data_Len			equ	3+Enemy_Move_Num	;Length of individual enemy definition
Enemy_Table_Len			equ	1+(Max_Enemy_Num*Enemy_Data_Len)	;Length of standard tables
Fragment_Table_Len		equ	Max_Fragment_Num*3
Canister_Table_Len		equ	Max_Canister_Num*5
Dolphin_Table_Len		equ	2+Dolphin_Move_Num
Screen_Len              equ 3*Map_Len+4+Enemy_Table_Len+Dolphin_Table_Len ;Length of data for each screen
Copy_Addr  				equ $80f0       ;Copy address for current screen       
Curr_Dir_Table  		equ Copy_Addr	;Pointer to the direction table for the current screen
Curr_Enemies			equ	Copy_Addr+4	;and the enemy table
Curr_Dolphin			equ	Copy_Addr+4+Enemy_Table_Len	;and the dolphin move table
Curr_Back1				equ	Copy_Addr+4+Enemy_Table_Len+Dolphin_Table_Len	;and the tilemap layers
Curr_Back2				equ	Copy_Addr+4+Enemy_Table_Len+Dolphin_Table_Len+Map_Len
Curr_Back3				equ	Copy_Addr+4+Enemy_Table_Len+Dolphin_Table_Len+(2*Map_Len)
Vector_Table_Addr       equ $c200	;$ba00       ;Start of IM2 vector table
ISR_Addr                equ $bebe	;$bbbb       ;MSB and LSB of IM2 ISR
Temp_Canister_Text		equ	$c415		;Area where data canister text is copied to be displayed
Temp_Copy_Addr			equ	$8300		;Area where data is copied to be decompressed
Mirrored_Sprite_Dest	equ	$b900		;Area where mirrored enemy sprites are copied
Obj_Table				equ	$b100		;Area where object table for level is copied
Gfx_Addr				equ	$b300		;Area where sprite data is decompressed
Tileset_Addr			equ	$9740		;Area where tileset data is decompressed
Gfx_Buffer_1			equ	$8f00		;1024	;Used for double-buffering of sub/other sprites
Gfx_Buffer_2			equ	$9300		;1024
Back_Buffer_1			equ	$8300		;1024	;Background image
Back_Buffer_2			equ	$8700		;1024
Back_Mask				equ	$8b00		;1024
Title_Menu_1_Pos_Y      equ 44          ;Cursor positions of title menu items
Title_Menu_2_Pos_Y      equ 50
Title_Menu_3_Pos_Y      equ 56
Title_Menu_Pos_X        equ 46
Credits_Pos_X           equ 6			;X position of cursor for credit screen
Credits_Pos				equ	$0100+Credits_Pos_X	;Cursor position of start of credit text
Lose_Pos				equ	$1520		;Cursor position of lose text
Sub_Height				equ	10			;Sort of self-explanatory...
Sub_Width				equ	16
Ink_Width				equ	8
Ink_Height				equ	8
Ink_Cloud_Height		equ	16
Proj_Width				equ	8
Object_Width			equ	8
Object_Height			equ	6
Dolphin_Height			equ	9
Dolphin_Width			equ	24
Screen_Width			equ	16			;Width of screen in bytes
Screen_Height_P			equ	64			;Screen dimensions in pixels
Screen_Height			equ	64
Screen_Width_P			equ	128
Status_Pos				equ	$3800		;Position of status display
Window_Height			equ	40			;Dimensions of standard window
Window_Width			equ	12
Window_Y				equ	10
Window_Offset			equ	Window_Y*Screen_Width+((Screen_Width-Window_Width)/2)
Window_Text_Pos			equ	(Window_Y+2)*256+((Screen_Width-Window_Width)*4)+2
Window_Pos				equ	Window_Y*256+((Screen_Width-Window_Width)*4)
Station_Door_Tile1		equ	$a9			;Tiles with station door
Station_Door_Tile2		equ	$ad


.org _asm_exec_ram

    nop
    jp Prog_Start

    .dw 0,Shell_Title

Shell_Title: .db "Operation Neptune",0

Prog_Start:
	set 1,(iy+$16)						;(table needs to be recalculated)
	ld (End_Game_SP),sp					;Save stack pointer
    call _runindicoff					;Normal init
    call _flushallmenus
	xor a								;No projectiles of any kind yet
	ld (Ink_State),a
	ld (Proj_State),a
	ld (Enemy_Num),a					;or enemies, either
	ld (Draw_Status),a					;Reset all the other necessary vars
	ld (Sub_Flash),a
	ld (Object_Anim_Enable),a
	ld (Dolphin_Enable),a
	ld hl,Tile_Str_Name-1				;Make sure that the string w/tileset data exists
	rst 20h
	rst 10h
	ret c
	call _ex_ahl_bde					;then copy it to the temp area
	call _AHL_PLUS_2
	call _SET_ABS_SRC_ADDR
	ex de,hl
	call _DATA_SIZE_TO_DE
	ex de,hl
	xor a
	call _SET_MM_NUM_BYTES
	ld a,$01
	ld hl,Temp_Copy_Addr-$8000
	call _SET_ABS_DEST_ADDR
	call _MM_LDIR
	ld hl,Temp_Copy_Addr				;then decompress it
	ld de,Tileset_Addr
	call Decompress
	ld hl,Gfx_Str_Name-1				;Do the same for the string w/sprites
	rst 20h
	rst 10h
	ret c
	call _ex_ahl_bde
	call _AHL_PLUS_2
	call _SET_ABS_SRC_ADDR
	ex de,hl
	call _DATA_SIZE_TO_DE
	ex de,hl
	xor a
	call _SET_MM_NUM_BYTES
	ld a,$01
	ld hl,Temp_Copy_Addr-$8000
	call _SET_ABS_DEST_ADDR
	call _MM_LDIR
	ld hl,Temp_Copy_Addr
	ld de,Gfx_Addr
	call Decompress
	call Mirror_Sprites					;Create reversed copies of all the enemy sprites

#include "title.asm"					;Show title screen, etc.

Reset_Game:
	di
    ld hl,Level_Str_Name-1				;Load the level string,
    rst 20h
    rst 10h
    ret c
	ld a,b
	ex de,hl
	call _AHL_PLUS_2
	call _SET_ABS_SRC_ADDR
	xor a
	ld hl,Curr_Zone						;and copy the zone name,
	call _SET_ABS_DEST_ADDR
	ld hl,16
	call _MM_LDIR_SET_SIZE
	call _LOAD_ABS_SRC_ADDR
	call _get_word_ahl
	ld b,a
	ld a,e
	ld (Curr_Sector),a					;the current sector number,
	ld a,b
	call _get_word_ahl
	ld (Sub_Init_Pos),de				;the sub's initial position,
	call _get_word_ahl
	push af
	push hl
	ld a,d
	ld (Num_Objects),a					;the number of objects,
	ld h,e
	ld l,Fragment_Table_Len+Canister_Table_Len+1
;CHECK HERE IF >MAX NUMBER OF SCREENS (30)
	call _htimesl						;(find length of object table)
	xor a
	call _SET_MM_NUM_BYTES				;and the object table
	ex de,hl
	inc a
	ld hl,Obj_Table-$8000
	call _SET_ABS_DEST_ADDR
	pop hl
	pop af
	call _SET_ABS_SRC_ADDR
	add hl,de
	adc a,0
	ld (Level_Start_Addr),a
	ld (Level_Start_Addr+1),hl			;then save the starting address of the screens
	call _MM_LDIR
	ei
	ld a,Oxygen_Start					;Init oxygen
	ld (Oxygen),a
;RESET OBJECT TABLE (IF RESET, NOT RETURN TO TITLE)
Start_Game:								;Start of game!!!
    ld de,(Sub_Init_Pos)                ;Set initial position of sub
    ld (Sub_Pos),de
	ld hl,sub_right						;and initial sprite
    ld (Sub_Sprite),hl
	xor a
	ld (Sub_Dir),a						;and init vars
	ld (Last_Move),a
	ld (Ink_State),a
	ld (Proj_State),a
	inc a
	ld (Object_Anim_Enable),a
	ld (Dolphin_Enable),a
	ld (New_Level),a
	ld a,Ink_Start						;and ink pellets
	ld (Num_Ink_Pellets),a
	ld hl,Sub_Vert_Delay_1				;and sub speed
	ld (Sub_Vert_Delay_Coeff),hl
	ld hl,Sub_Horiz_Delay_1
	ld (Sub_Horiz_Delay_Coeff),hl
	call Wait_For_No_Key
	ld a,1
	call Load_New_Screen				;then start at screen 1
	jp Draw_Screen


Next_Level:
	ld sp,(End_Game_SP)					;(restore sp first)
	ld a,(Num_Objects)					;Check if any objects are left
	or a
	jr z,Next_Level_Continue			;If they've all been collected, then continue to the next level
	call Draw_Window					;Otherwise, show a message,
	ld hl,More_Objects_Text
	ld b,More_Objects_Text_Lines
	call Draw_Window_Text
	call Copy_From_Buffer
	call Wait_For_A_Key
	call Wait_For_No_Key
	call Erase_Window
	ld hl,Sub_Vert_Delay_1				;reset acceleration,
	ld (Sub_Vert_Delay_Coeff),hl
	ld hl,Sub_Horiz_Delay_1
	ld (Sub_Horiz_Delay_Coeff),hl
	xor a
	ld (Last_Move),a
	jp Draw_Sub							;and return
Next_Level_Continue:
	call Draw_Window					;Draw window with message,
	ld hl,Next_Level_Text
	ld b,Next_Level_Text_Lines
	call Draw_Window_Text
	call Copy_From_Buffer
	call Wait_For_A_Key
	call Wait_For_No_Key
	jp Reset_Game						;and start the next level!


End_Game:
	ld hl,Lose_Text						;Show "lose" text
	ld de,Lose_Pos
	ld (_penCol),de
	ld a,$fc
	ld (fvputc_vidplane),a
	call fvputs
	ld hl,Lose_Text
	ld de,Lose_Pos
	ld (_penCol),de
	ld a,$ca
	ld (fvputc_vidplane),a
	call fvputs
	call Wait_For_A_Key					;Wait for a key to be pressed
;	jr Back_To_Title					;and return to the title screen

Back_To_Title:
	xor a								;Kill ink pellets
	ld (Ink_State),a
	ld (Enemy_Num),a					;and enemies
	ld (Proj_State),a					;and all other stuff that could screw the mem
	ld (Draw_Status),a
	ld (Sub_Flash),a
	ld (Object_Anim_Enable),a
	call Draw_Title_Screen				;and go back to title screen
	call Wait_For_No_Key
	jp Title_Menu

Exit_Prog:
	call Wait_For_No_Key
    res textInverse,(iy+textflags)
    call Close_Gray
	jp _flushallmenus

#include "mainloop.asm"					;Main loop of game with keys, etc.
#include "move.asm"						;Move code for sub/ink pellet/fish etc.
#include "ink.asm"						;Ink pellet-related code
#include "enemies.asm"					;Enemy code
#include "enmblt.asm"					;Enemy projectile code
#include "dolphin.asm"					;Dolphin code
#include "objects.asm"					;Object code
#include "status.asm"					;Status display code
#include "routines.asm"					;Routines
#include "fvputs.inc"					;Joshua Seagoe's amazing text routine
#include "data.asm"						;Data

int_copy:
.org ISR_Addr

int_start:
	exx
	ex af,af'
	in a,(3)
	bit 3,a
	jr z,no_down_left_bug
	and %11111110
	out (3),a
no_down_left_bug:
	bit 1,a
	jr z,end_int
	ld hl,gray_cntr
	inc (hl)
	ld a,(hl)
	cp 3
	ld a,$3c	;$3c
	jr c,set_video_port
	ld a,$0a	;$0a
	ld (hl),0
set_video_port:
	out (0),a
end_int:
	call Move_Ink_Pellet				;Move ink pellet if necessary
	call Move_Enemies					;Move enemies if necessary
	call Move_Enm_Proj					;(and their projectiles)
	call Move_Dolphin					;along with the dolphin, if necessary
	call Draw_Status_Delay				;then handle oxygen display delay
	call Anim_Objects					;and animate the objects
	call Sub_Flash_Delay				;and flash the sub, if needed
;Next section from ISR template by Matt Johnson
    in a,(3)                            ; Get byte from Port 3
    rra                                 ; Check Bit 0 (ON interrupt status) by using carry
    ld  a,9
    adc a,0                             ; A = $09 if ON interrupt did not occur, $0A if (carry) so
    out (3),a                           ; Write either $09 or $0A to Port 3
    ld a,$0B                                                        
    out (3),a                           ; Write $0B to Port 3
    ex af,af'
    exx
    ei
    reti

gray_cntr: .db 1

int_end:

.end