;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Oxygen-supply dolphin code/data


;Dolphin data table format

;(0)-Status of dolphin
;(1-2)-Starting position
;(3-22)-Movements

;Movement format

;(bit)
;0-up
;1-down
;2-left
;3-right
;$ff = end of sequence

;Status byte format

;bit 0 set if dolphin exists
;bit 1 set if dolphin has oxygen tank
;bit 2 set if dolphin is going right, clear if going left
;bit 3 set if dolphin is on frame 2, clear if on frame 1

bitDExists		equ		0
bitDHasTank		equ		1
bitDMoveDir		equ		2
bitDAnimState	equ		3

Dolphin_SPos	equ		Curr_Dolphin
Dolphin_MoveTbl	equ		Curr_Dolphin+2

;//////////////////////////////Initialization//////////////////////////////

Init_Dolphin:
	ld hl,(Dolphin_SPos)				;Init position
	ld (Dolphin_Pos),hl
	ld (Old_Dolphin_Pos),hl
	ld hl,Dolphin_MoveTbl				;pointer to move table
	ld (Dolphin_Move),hl
	ld a,Dolphin_Move_Delay				;delay counter
	ld (Dolphin_Move_Counter),a
	ld a,Dolphin_Next_Move_Delay
	ld (Dolphin_Next_Move_Counter),a
	ld a,1								;and show that it needs to be redrawn
	ld (Redraw),a
	ret

;//////////////////////////////Move//////////////////////////////

Move_Dolphin:
	ld a,(Dolphin_Enable)
	or a
	ret z
	ld hl,(Curr_Dolphin_Status)			;If there isn't a dolphin, we don't need to move it
	bit bitDExists,(hl)
	ret z
	ld hl,Dolphin_Move_Counter			;From here on, it's just about the same as the enemy move code (but much simpler!)...
	dec (hl)
	ret nz
	ld (hl),Dolphin_Move_Delay
	ld a,1
	ld (Redraw),a
	ld de,(Dolphin_Move)
Move_Dolphin_Process:
	ld a,(de)
	ld hl,Dolphin_Next_Move_Counter
	dec (hl)
	jr nz,Move_Dolphin_Process_Move
	ld (hl),Dolphin_Next_Move_Delay
	inc de
	ld (Dolphin_Move),de
Move_Dolphin_Process_Move:
	ld hl,(Curr_Dolphin_Status)
	bit bitDAnimState,(hl)
	jr nz,$+6
	set bitDAnimState,(hl)
	jr $+4
	res bitDAnimState,(hl)
	cp $ff
	jr z,Move_Dolphin_Reset
	or a
	ret z
	ld b,a
	ld hl,(Dolphin_Pos)
	ld (Old_Dolphin_Pos),hl
	ld de,Dolphin_Pos
	ld hl,(Curr_Dolphin_Status)
	ld a,b
	bit bitUp,a
	call nz,Move_Dolphin_Up
	ld a,b
	bit bitDown,a
	call nz,Move_Dolphin_Down
	ld a,b
	bit bitLeft,a
	call nz,Move_Dolphin_Left
	ld a,b
	bit bitRight,a
	call nz,Move_Dolphin_Right
	ret
Move_Dolphin_Reset:
	ld hl,Dolphin_Next_Move_Counter
	inc (hl)
	ld de,Dolphin_MoveTbl
	ld (Dolphin_Move),de
	jr Move_Dolphin_Process
Move_Dolphin_Up:
	inc de
	ld a,(de)
	sub Dolphin_Move_Dist
	ld (de),a
	dec de
	ret
Move_Dolphin_Down:
	inc de
	ld a,(de)
	add a,Dolphin_Move_Dist
	ld (de),a
	dec de
	ret
Move_Dolphin_Left:
	ld a,(de)
	sub Dolphin_Move_Dist
	ld (de),a
	res bitDMoveDir,(hl)
	ret
Move_Dolphin_Right:
	ld a,(de)
	add a,Dolphin_Move_Dist
	ld (de),a
	set bitDMoveDir,(hl)
	ret


;//////////////////////////////Draw//////////////////////////////

Erase_Dolphin:
	ld hl,(Curr_Dolphin_Status)
	bit bitDExists,(hl)
	ret z
	ld a,Dolphin_Height
	ld de,(Old_Dolphin_Pos)
	jp Restore_Gray_Sprite

Draw_Dolphin:
	ld hl,(Curr_Dolphin_Status)
	ld a,(hl)
	bit bitDExists,a
	ret z
	and %00001110
	ld l,a
	ld h,0
	ld de,Dolphin_Sprite_Table
	add hl,de
	call _ldHLind
	ld bc,(Dolphin_Pos)
	jp Draw_Sprite

.end