;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Enemy code/data


;Enemy data table format

;0-Number of enemies
;=====================(5 individual enemy tables)
;(0)-Type of enemy
;(1-2)-Starting position
;(3-22)-Movements

;Movement format

;(bit)
;0-up
;1-down
;2-left
;3-right
;4-fire
;5-force to frame 1
;6-force to frame 2
;$ff = end of sequence

;Enemy types

;0-angel fish
;1-sea anemone
;2-man-of-war
;3-puffer fish
;4-octopus
;5-squid
;6-barracuda
;7-viper fish
;8-garden eel
;9-mega mouth (?)
;10-crab
;11-clam
;12-sea hopper
;13-hammerhead shark
;14-stingray
;15-barnacle
;bit 7 set if enemy is in ink cloud
;bit 6 set if enemy doesn't move
;bit 5 set if enemy is on alternate frame (if animated)
;bit 4 clear if enemy is going left, bit 4 set if enemy is going right

enmAngelFish	equ		0
enmAnemone		equ		1
enmManOfWar		equ		2
enmPufferFish	equ		3
enmOctopus		equ		4
enmSquid		equ		5
enmBarracuda	equ		6
enmViperFish	equ		7
enmGardenEel	equ		8
enmMegaMouth	equ		9
enmCrab			equ		10
enmClam			equ		11
enmSeaHopper	equ		12
enmHammerhead	equ		13
enmStingray		equ		14
enmBarnacle		equ		15

bitUp			equ		0
bitDown			equ		1
bitLeft			equ		2
bitRight		equ		3
bitFire			equ		4
bitAnim1		equ		5
bitAnim2		equ		6

bitInCloud		equ		7
bitNoMove		equ		6
bitAnimState	equ		5
bitMoveDir		equ		4

Enemy_Num		equ		Curr_Enemies
Enemy_1_Type	equ		Curr_Enemies+1
Enemy_1_SPos	equ		Curr_Enemies+2
Enemy_1_MoveTbl	equ		Curr_Enemies+4
Enemy_2_Type	equ		Curr_Enemies+24
Enemy_2_SPos	equ		Curr_Enemies+25
Enemy_2_MoveTbl	equ		Curr_Enemies+27
Enemy_3_Type	equ		Curr_Enemies+47
Enemy_3_SPos	equ		Curr_Enemies+48
Enemy_3_MoveTbl	equ		Curr_Enemies+50
Enemy_4_Type	equ		Curr_Enemies+70
Enemy_4_SPos	equ		Curr_Enemies+71
Enemy_4_MoveTbl	equ		Curr_Enemies+73
Enemy_5_Type	equ		Curr_Enemies+93
Enemy_5_SPos	equ		Curr_Enemies+94
Enemy_5_MoveTbl	equ		Curr_Enemies+96

;//////////////////////////////Initialization//////////////////////////////

Init_Enemies:
	ld hl,(Enemy_1_SPos)				;Init positions of enemies
	ld (Enemy_1_Pos),hl
	ld (Old_Enemy_1_Pos),hl
	ld hl,(Enemy_2_SPos)
	ld (Enemy_2_Pos),hl
	ld (Old_Enemy_2_Pos),hl
	ld hl,(Enemy_3_SPos)
	ld (Enemy_3_Pos),hl
	ld (Old_Enemy_3_Pos),hl
	ld hl,(Enemy_4_SPos)
	ld (Enemy_4_Pos),hl
	ld (Old_Enemy_4_Pos),hl
	ld hl,(Enemy_5_SPos)
	ld (Enemy_5_Pos),hl
	ld (Old_Enemy_5_Pos),hl
	ld hl,Enemy_1_MoveTbl				;pointers to move tables
	ld (Enemy_1_Move),hl
	ld hl,Enemy_2_MoveTbl
	ld (Enemy_2_Move),hl
	ld hl,Enemy_3_MoveTbl
	ld (Enemy_3_Move),hl
	ld hl,Enemy_4_MoveTbl
	ld (Enemy_4_Move),hl
	ld hl,Enemy_5_MoveTbl
	ld (Enemy_5_Move),hl
	ld a,Enemy_Move_Delay				;delay counter
	ld (Enemy_Move_Counter),a
	ld a,Enemy_Next_Move_Delay
	ld (Enemy_Next_Move_Counter),a
	ld a,1								;and show that they need to be redrawn
	ld (Redraw),a
	ret

;//////////////////////////////Move//////////////////////////////

Move_Enemies:
	ld a,(Enemy_Num)					;If there aren't any enemies, we don't need to move them!
	or a
	ret z
	ld hl,Enemy_Move_Counter			;Decrement enemy move counter
	dec (hl)
	ret nz								;If it's not at zero yet, return
	ld b,a
	ld (hl),Enemy_Move_Delay			;If it has timed out, reset the counter
	ld hl,Enemy_1_Move					;Start at beginning of enemy table
	ld (Enemy_Data_Ptr),hl
	ld hl,Enemy_1_Type
	ld (Enemy_Data_Ptr2),hl
	ld hl,Enemy_1_Pos
	ld (Enemy_Pos_Ptr),hl
	ld hl,Old_Enemy_1_Pos
	ld (Enemy_Old_Pos_Ptr),hl
Move_Enemy_Loop:						;and move 'em all
	push bc
	ld hl,(Enemy_Data_Ptr2)				;Check current enemy
	bit bitInCloud,(hl)					;Is it covered by ink cloud?
	jp nz,Move_Enemy_End				;If it is, skip moving this one
	bit bitNoMove,(hl)					;Also skip it if it's not supposed to move
	jp nz,Move_Enemy_End
	ld hl,(Enemy_Data_Ptr)
Move_Enemy_Process:
	ld e,(hl)
	inc hl
	ld d,(hl)
	ld a,(de)							;Get the direction the current enemy is supposed to move
	push hl
	ld hl,Enemy_Next_Move_Counter		;See if we can go to the next movement yet
	dec (hl)
	jr nz,Move_Enemy_Process_Move		;If not, keep on processing the same movement
	ld (hl),Enemy_Next_Move_Delay
	pop hl
	inc de								;Otherwise, increment the pointer in the movement table
	ld (hl),d							;and update the pointer
	dec hl
	ld (hl),e
	jr $+3
Move_Enemy_Process_Move:
	pop hl
	cp $ff								;If it's the end of the sequence,
	jr z,Move_Enemy_Reset				;then go back to the beginning
	or a								;If there isn't any movement,
	jp z,Move_Enemy_End					;then check next enemy
	ld b,a
	ld hl,(Enemy_Pos_Ptr)				;Otherwise, set current position as old position
	ld e,(hl)
	inc hl
	ld d,(hl)
	ld hl,(Enemy_Old_Pos_Ptr)
	ld (hl),e
	inc hl
	ld (hl),d
	ld de,(Enemy_Pos_Ptr)
	ld hl,(Enemy_Data_Ptr2)
	ld a,b
	bit bitUp,a							;and find which direction(s) it should be moving
	call nz,Move_Enemy_Up
	ld a,b
	bit bitDown,a
	call nz,Move_Enemy_Down
	ld a,b
	bit bitLeft,a
	call nz,Move_Enemy_Left
	ld a,b
	bit bitRight,a
	call nz,Move_Enemy_Right
	ld a,b
	bit bitAnim1,a						;Check for change in animation states
	call nz,Move_Enemy_Anim1
	bit bitAnim2,a
	call nz,Move_Enemy_Anim2
	bit bitFire,a
	call nz,Move_Enemy_Fire
	jr Move_Enemy_End					;When done moving, go to next enemy
Move_Enemy_Reset:
	ld hl,Enemy_Next_Move_Counter		;Reset move counter
	inc (hl)
	ld hl,(Enemy_Data_Ptr)				;then set move position pointer
	ld de,(Enemy_Data_Ptr2)				;to beginning of table
	inc de
	inc de
	inc de
	ld (hl),e
	inc hl
	ld (hl),d
	dec hl
	jr Move_Enemy_Process				;and now process the first movement again
Move_Enemy_Up:
	inc de
	ld a,(de)
	sub Enemy_Move_Dist
	ld (de),a
	dec de
	ret
Move_Enemy_Down:
	inc de
	ld a,(de)
	add a,Enemy_Move_Dist
	ld (de),a
	dec de
	ret
Move_Enemy_Left:
	ld a,(de)
	sub Enemy_Move_Dist
	ld (de),a
	res bitMoveDir,(hl)
	ret
Move_Enemy_Right:
	ld a,(de)
	add a,Enemy_Move_Dist
	ld (de),a
	set bitMoveDir,(hl)
	ret
Move_Enemy_Fire:
	ld a,(Enemy_Next_Move_Counter)		;(make sure the enemy only fires on the first cycle)
	cp Enemy_Next_Move_Delay
	ret nz
	bit bitMoveDir,(hl)					;Find the direction the enemy is moving
	jr z,Move_Enemy_Fire_Left
Move_Enemy_Fire_Right:
	ld b,1
	jp Fire_Enm_Proj					;then fire the projectile
Move_Enemy_Fire_Left:
	ld b,2
	jp Fire_Enm_Proj
Move_Enemy_Anim1:
	res bitAnimState,(hl)
	ret
Move_Enemy_Anim2:
	set bitAnimState,(hl)
	ret	
Move_Enemy_End:
	call Move_Enemy_UpdatePtrs			;Update pointers
	pop bc
	dec b								;and loop again until all enemies have been moved
	jp nz,Move_Enemy_Loop
	ld a,1
	ld (Redraw),a						;When done, indicate that redraw is needed
	ret

Move_Enemy_UpdatePtrs:
	ld hl,(Enemy_Data_Ptr2)				;Update pointers
	ld de,Enemy_Data_Len
	add hl,de
	ld (Enemy_Data_Ptr2),hl
	ld hl,(Enemy_Data_Ptr)
	inc hl
	inc hl
	ld (Enemy_Data_Ptr),hl
	ld hl,(Enemy_Pos_Ptr)
	inc hl
	inc hl
	ld (Enemy_Pos_Ptr),hl
	ld hl,(Enemy_Old_Pos_Ptr)
	inc hl
	inc hl
	ld (Enemy_Old_Pos_Ptr),hl
	ret


;//////////////////////////////Draw//////////////////////////////

Erase_Enemies:
	ld a,(Enemy_Num)					;Look at the number of enemies
	or a
	ret z								;If there aren't any, skip this
	ld b,a
	ld hl,Old_Enemy_1_Pos
	ld (Alt_Enemy_Old_Pos_Ptr),hl
	ld hl,Enemy_1_Type
	ld (Alt_Enemy_Data_Ptr),hl
Erase_Enemy_Loop:
	push bc
	ld hl,(Alt_Enemy_Old_Pos_Ptr)
	ld e,(hl)							;Get the enemy's position
	inc hl
	ld d,(hl)
	inc hl
	ld (Alt_Enemy_Old_Pos_Ptr),hl
	ld hl,(Alt_Enemy_Data_Ptr)
	ld a,(hl)
	ld bc,Enemy_Data_Len
	add hl,bc
	ld (Alt_Enemy_Data_Ptr),hl
	bit bitInCloud,a					;(if it's in an ink cloud, it doesn't have to be erased)
	jr nz,Erase_Enemy_Loop_Skip
	and $0f
	push de
	call Find_Enemy_Sprite
	pop de
	inc hl
	ld a,(hl)
	call Restore_Gray_Sprite			;and erase it
Erase_Enemy_Loop_Skip:
	pop bc								;then move to the next enemy
	djnz Erase_Enemy_Loop
	ret

Draw_Enemies:
	ld a,(Enemy_Num)					;Find number of enemies to draw
	ld b,a
	or a								;(if there aren't any, return)
	ret z
	ld hl,Enemy_1_Type					;Start at beginning of enemy table
	ld (Alt_Enemy_Data_Ptr),hl
	ld hl,Enemy_1_Pos
	ld (Alt_Enemy_Pos_Ptr),hl
Draw_Enemy_Loop:						;and draw them all!!!
	push bc
	ld hl,(Alt_Enemy_Data_Ptr)
	ld a,(hl)							;Look at enemy type
	bit bitInCloud,a					;Is it being covered by an ink cloud?
	jr nz,Draw_Enemy_End				;If so, skip drawing this enemy
	call Find_Enemy_Sprite
	push hl								;Save that address
	ld hl,(Alt_Enemy_Pos_Ptr)			;Get position enemy should be drawn at
	call _ldHLind
	ld c,l
	ld b,h
	pop hl								;restore sprite address
	call Draw_Sprite					;and draw it
Draw_Enemy_End:
	ld hl,(Alt_Enemy_Data_Ptr)			;Now update pointers
	ld de,Enemy_Data_Len
	add hl,de
	ld (Alt_Enemy_Data_Ptr),hl
	ld hl,(Alt_Enemy_Pos_Ptr)
	inc hl
	inc hl
	ld (Alt_Enemy_Pos_Ptr),hl
	pop bc
	djnz Draw_Enemy_Loop				;Loop again if we haven't drawn all enemies
	ret


.end