;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Enemy projectile code

projNo		equ		0
projRight	equ		1
projLeft	equ		2
projErase	equ		3

;//////////////////////////////Init////////////////////////////////

Fire_Enm_Proj:
	push hl
	ex de,hl								;Get enemy's position,
	call _ldHLind
	ld a,Proj_Move_Delay					;init move delay,
	ld (Proj_Move_Counter),a
	ld a,b
	ld (Proj_State),a						;set direction of projectile,
	dec a
	jr z,Fire_Enm_Proj_Right
Fire_Enm_Proj_Left:							;and calculate starting position of ink pellet depending on direction of sub
;FIX POS DEPENDING ON ENEMY
	ld a,l
	sub Proj_Width
	jr Fire_Enm_Proj_End
Fire_Enm_Proj_Right:
;FIX POS DEPENDING ON ENEMY
	ld a,l
	add a,8
Fire_Enm_Proj_End:
	ld l,a
	ld (Proj_Pos),hl
	ld (Old_Proj_Pos),hl
	pop hl									;then find the sprite for the projectile from the enemy type
	ld a,(hl)
	and $0f
	add a,a
	ld e,a
	ld d,0
	ld hl,Enm_Proj_Sprite_Table
	add hl,de
	call _ldHLind
	ld (Proj_Sprite),hl
	ret

;//////////////////////////////Move//////////////////////////////

Move_Enm_Proj:
	ld a,(Proj_State)						;Is there currently a projectile being fired?
	or a
	ret z									;If not, end this routine
	ld hl,Proj_Move_Counter					;Otherwise, decrement move counter
	dec (hl)
	ret nz									;Skip if it's not at zero yet
	ld b,Proj_Move_Delay					;If it is, reset counter and move
	ld (hl),b
	cp projErase							;(unless it's supposed to be erased - in that case, just skip the rest of this crap)
	ret z
	ld hl,(Proj_Pos)						;Update position
	ld (Old_Proj_Pos),hl
	ld a,l									;then look at x coord
	or a
	jp z,Kill_Enm_Proj						;If it's gone off the left edge of the screen, erase it
	cp Screen_Width_P-8						;same with the right side of the screen
	jp z,Kill_Enm_Proj
	ex de,hl
;FINISH LATER, GET REAL SIZE FOR PROJECTILE INSTEAD OF 8
	ld bc,8*256+8
	call Test_Hit_Sub						;Check if projectile has hit sub
	jr nc,Move_Enm_Proj_Test_Hit_Skip
	call Dec_Lives							;If it has, remove one unit of oxygen
	jr Kill_Enm_Proj						;and erase projectile
Move_Enm_Proj_Test_Hit_Skip:
	ld hl,Proj_Pos_X						;If it hasn't gone off the edge of the screen or hit an enemy, it needs to be moved
	ld a,(Proj_State) 						;Check which direction it's supposed to be moving in
	dec a
	jr z,Move_Enm_Proj_Right
Move_Enm_Proj_Left:
	dec (hl)								;Sorta obvious...if it's moving left, dec the x pos
	jr $+3
Move_Enm_Proj_Right:
	inc (hl)								;and if it's going right, inc the x pos
	ld a,1									;Show that a redraw is needed
	ld (Redraw),a
    ld de,(Proj_Pos)
	call Test_Hit_Wall						;Find if it's hit a wall
	ret nc									;If not, return
											;If it has,
Kill_Enm_Proj:
	ld a,projErase							;set the projectile state to 'erase'
	ld (Proj_State),a
	ld a,1									;and show a redraw is needed
	ld (Redraw),a
	ret

;//////////////////////////////Draw//////////////////////////////

Erase_Enm_Proj:
	ld a,(Proj_State)					;No complex code here...just make sure the enemy projectile exists
	or a
	ret z
	ld hl,(Proj_Sprite)
	inc hl
	ld a,(hl)
	ld de,(Old_Proj_Pos)
	jp Restore_Gray_Sprite

Draw_Enm_Proj:
	ld a,(Proj_State)					;Is there an enemy projectile?
	or a
	ret z								;If not, return
	ld hl,(Proj_Sprite)					;Otherwise, draw it
	cp projErase
	jr z,Draw_Enm_Proj_Erase			;If it needs to be erased, then do that
	ld bc,(Proj_Pos)
	jp Draw_Sprite
Draw_Enm_Proj_Erase:
	inc hl								;find height
	ld a,(hl)
	ld de,(Proj_Pos)					;erase projectile
	call Restore_Gray_Sprite
	xor a								;and show that it doesn't exist anymore
	ld (Proj_State),a
	ret


.end