;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Ink pellet code


;ink_state values
inkNone		equ			0
inkRight	equ			1
inkLeft		equ			2
inkSplat	equ			3
inkCloud	equ			4
inkErase	equ			5
inkEraseThenCloud	equ	6

;//////////////////////////////Init////////////////////////////////

Fire_Ink_Pellet:
	ld a,(Ink_State)						;Is there already an ink pellet being fired?
	or a
	ret nz									;If so, go back to key loop - we can't handle two at once!
	call Start_Show_Status					;(start the status display to show the number of pellets left)
	ld a,(Num_Ink_Pellets)
	or a									;Otherwise, check if there are any remaining ink pellets
	ret z
	dec a
	ld (Num_Ink_Pellets),a
	ld a,Ink_Move_Delay						;If there are, init move delay,
	ld (Ink_Move_Counter),a
	ld a,(Sub_Dir)							;set direction of ink pellet depending on direction of sub,
	inc a
	ld (Ink_State),a
	dec a
	jr z,Fire_Ink_Pellet_Right
Fire_Ink_Pellet_Left:						;and calculate starting position of ink pellet depending on direction of sub
	ld de,(Sub_Pos)
	ld a,e
	cp Ink_Width							;(make sure the ink pellet doesn't go off the screen)
	jr nc,$+4
	ld a,Ink_Width
	sub Ink_Width
	jr Fire_Ink_Pellet_End
Fire_Ink_Pellet_Right:
	ld de,(Sub_Pos)
	ld a,e
	add a,Sub_Width
	cp Screen_Width_P-Ink_Width
	jr c,$+4
	ld a,Screen_Width_P-Ink_Width
Fire_Ink_Pellet_End:
	ld e,a
	ld (Ink_Pos),de
	ld (Old_Ink_Pos),de
	ret

;//////////////////////////////Move//////////////////////////////

Move_Ink_Pellet:
	ld a,(Ink_State)						;Is there currently an ink pellet being fired?
	or a
	ret z									;If not, skip this
	ld hl,Ink_Move_Counter					;Otherwise, decrement ink move counter
	dec (hl)
	ret nz									;Skip if it's not at zero yet
	ld (hl),Ink_Move_Delay					;If it is, reset counter and move ink pellet
	cp inkSplat								;By this point we know that there is an ink pellet - is it a splat?
	jp z,Move_Ink_Pellet_Splat_Delay		;If so, do delay before erasing it
	cp inkCloud								;Is it an ink cloud?
	jp z,Move_Ink_Pellet_Cloud_Delay		;If so, do another delay before erasing it
	cp inkErase
	ret z
	cp inkEraseThenCloud
	ret z
	ld hl,(Ink_Pos)							;Otherwise, it's moving
	ld (Old_Ink_Pos),hl						;First, save its position
	ex de,hl
	ld bc,Ink_Height*256+Ink_Width
	ld hl,ink_pellet
	call Test_Hit_Enemy						;Check if ink pellet has hit enemy
	jr nc,Move_Ink_Pellet_Test_Enemy_Skip
	ld hl,(Enemy_Data_Ptr)					;If it has, show that enemy has been hit,
	ld a,(hl)
	and $0f
	cp enmManOfWar							;(unless it's a man-of-war or a barnacle, which are resistant to ink)
	jr z,Kill_Ink_Pellet_Splat
	cp enmBarnacle
	jr z,Kill_Ink_Pellet_Splat
	set bitInCloud,(hl)
	ld hl,(Enemy_Pos_Ptr)					;set position of ink cloud to position of enemy that's been hit,
	call _ldHLind
	dec h
	dec h
	dec h
	dec l
	dec l
	dec l
	ld (Ink_Pos),hl
	jr Kill_Ink_Pellet_Cloud				;and draw ink cloud around enemy
Move_Ink_Pellet_Test_Enemy_Skip:
    ld de,(Ink_Pos)							;Now find if the ink pellet has hit any of the walls
	call Test_Hit_Wall
	jr c,Kill_Ink_Pellet_Splat				;If it has, show splat
	ld hl,Ink_Pos_X
	ld a,(hl)								;Look at x coord
	or a
	jr z,Move_Ink_Pellet_Edge				;If it's gone off the left edge of the screen, make it disappear
	cp Screen_Width_P-Ink_Width				;same with the right side of the screen
	jr nc,Move_Ink_Pellet_Edge
											;If it hasn't gone off the edge of the screen or hit anything, it needs to be moved
	ld a,(Ink_State) 						;Check which direction it's supposed to be moving in
	dec a
	jr z,Move_Ink_Pellet_Right
Move_Ink_Pellet_Left:
	dec (hl)								;Sorta obvious...if it's moving left, dec the x pos
	jr $+3
Move_Ink_Pellet_Right:
	inc (hl)								;and if it's going right, inc the x pos
	ld a,1									;Show that a redraw is needed
	ld (Redraw),a
	ret
Move_Ink_Pellet_Edge:						;If the ink pellet at the edge of the screen,
	ld de,(Ink_Pos)							;we need to decide whether to just kill it
	call FindPixel							;or to show a splat, so test if it's against a wall or not
	ld de,Back_Mask
	add hl,de
	ld a,(hl)
	or a
	jr z,Kill_Ink_Pellet_Erase				;If there's nothing there, then just erase it
;	jr Kill_Ink_Pellet_Splat				;Otherwise, show the splat


Kill_Ink_Pellet_Splat:						;If ink pellet has hit wall,
	ld a,Ink_Splat_Delay					;init splat delay counter,
	ld (Splat_Counter),a
	ld a,inkSplat							;show that it's in a splat,
	ld hl,(Ink_Pos)
	ld (Old_Ink_Pos),hl						;and update the position
	jr Kill_Ink_Pellet
Kill_Ink_Pellet_Cloud:						;If ink pellet has hit enemy,
	ld hl,Ink_Cloud_Delay					;init cloud delay
	ld (Cloud_Counter),hl
	ld a,inkEraseThenCloud					;and show that a cloud should be drawn instead of a splat,
	jr Kill_Ink_Pellet						;and continue normally...the situation is under control...please stay calm...
Kill_Ink_Pellet_Erase:
    ld a,inkErase							;Show that there's no ink pellet anymore
Kill_Ink_Pellet:
	ld (Ink_State),a
	ld a,1									;and that it needs to be redrawn
	ld (Redraw),a
	ret

Move_Ink_Pellet_Splat_Delay:
	ld hl,Splat_Counter						;Decrement splat counter
	dec (hl)
	ret nz									;If it isn't at 0 yet, return
	jr Kill_Ink_Pellet_Erase				;If it is, kill ink pellet

Move_Ink_Pellet_Cloud_Delay:
	ld hl,(Cloud_Counter)					;Decrement ink cloud counter
	dec hl
	ld (Cloud_Counter),hl
	ld a,h
	or l
	ret nz									;If still not at 0, return
	ld a,(Enemy_Num)						;Otherwise, remove ink cloud from all enemies,
	ld b,a
	ld hl,Enemy_1_Type
	ld de,Enemy_Data_Len
Move_Ink_Pellet_Kill_Cloud:
	res bitInCloud,(hl)
	add hl,de
	djnz Move_Ink_Pellet_Kill_Cloud
	jr Kill_Ink_Pellet_Erase				;then set the cloud to be erased

;//////////////////////////////Draw//////////////////////////////

Erase_Ink:
	ld a,(Ink_State)					;Check the state of the ink
	or a
	ret z
	cp inkCloud
	jr z,Erase_Ink_Cloud
	cp inkEraseThenCloud
	jr z,Erase_Ink_Both
	cp inkErase
	jr nz,Erase_Ink_Normal
Erase_Ink_Cloud:
	ld a,Ink_Cloud_Height				;to find the vertical distance to erase
	jr Erase_Ink_End
Erase_Ink_Both:
	ld a,Ink_Height
	call Erase_Ink_End
	ld hl,(Ink_Pos)
	ld (Old_Ink_Pos),hl
	ld a,Ink_Cloud_Height
	jr Erase_Ink_End
Erase_Ink_Normal:
	ld a,Ink_Height
Erase_Ink_End:
	ld de,(Old_Ink_Pos)
	jp Restore_Gray_Sprite


Draw_Ink:
	ld hl,ink_pellet
	ld a,(Ink_State)
	or a
	ret z
	cp inkSplat							;(if it's a splat, draw that instead of a pellet)
	jr z,Draw_Ink_Splat
	cp inkCloud							;(if it's a cloud, draw that)
	jr z,Draw_Ink_Cloud
	cp inkEraseThenCloud
	jr z,Draw_Ink_EraseThenCloud
	cp inkErase
	jr nz,Draw_Ink_Draw
Draw_Ink_Erase:
	xor a								;(if it's supposed to be erased,
	ld (Ink_State),a					;DON'T redraw it, and kill the pellet)
	ret
Draw_Ink_EraseThenCloud:
	ld a,inkCloud						;(if it's supposed to be erased and a cloud drawn,
	ld (Ink_State),a					;just don't redraw it this time, and set a cloud to be
	ret									;drawn for the next redraw cycle)
Draw_Ink_Cloud:
	ld hl,ink_cloud
	jr Draw_Ink_Draw
Draw_Ink_Splat:
	ld hl,ink_splat
Draw_Ink_Draw:
	ld bc,(Ink_Pos)
	jp Draw_Sprite						;and draw the new one

.end