;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Main loop

Draw_Screen:                            ;Draw background
	xor a								;Kill any existing ink pellets
	ld (Ink_State),a
	ld (Proj_State),a					;or enemy projectiles
	ld (New_Screen),a
	inc a
	ld (Redraw),a
	ld hl,(Sub_Pos)						;Init old sub pos
	ld (Old_Sub_Pos),hl
	call Init_Objects					;and the objects on this screen
	call Init_Dolphin					;and the dolphin, if present
	call Clear_Back
	ld a,$77							;(mask of back tilemaps get copied to mask)
	ld (Draw_Mask_Patch),a
	ld hl,Curr_Back1					;Now draw the background
	call DrawMap
	ld hl,Curr_Back2
	call DrawMap
	xor a								;(but not the front layer)
	ld (Draw_Mask_Patch),a
	ld hl,Curr_Back3
	call DrawMap
    ld hl,Back_Buffer_2
    ld de,Back_Buffer_1
	call Copy_To_Buffers
	ld a,(New_Level)					;Is this a new level?
	or a
	call nz,Show_Level_Info				;If so, show the current zone and sector
	call Copy_From_Buffer
Draw_Sub:
	ld de,(Old_Sub_Pos)					;Erase the sub,
	ld a,Sub_Height
	call Restore_Gray_Sprite
	ld a,1								;then set it to be redrawn
	ld (Redraw),a
	call Sub_Delay						;and delay to keep from moving too fast
Loop:
	ld a,(Redraw)						;Check if enemies/ink need to be redrawn
	or a
	call nz,Redraw_All					;If so, redraw them
Key_Loop:								;Wait for keypress
    ld a,k_g1
    call Get_Keypress
    cp k_exit							;Is it exit?
    jp z,Back_To_Title					;If so, go back to title screen
    cp k_2nd							;Is it 2nd?
	call z,Fire_Ink_Pellet				;If so, fire ink pellet
	ld a,k_g6							;Is it one of the contrast keys?
	call Get_Keypress
	cp k_add
	call z,Contrast_Up
	cp k_sub
	call z,Contrast_Down
    ld a,k_g7							;Is it one of the arrow keys?
    call Get_Keypress
	ld hl,(Sub_Pos_X)					;(set the old position to the current position for when it's moved)
	ld (Old_Sub_Pos_X),hl
    push af
    ld de,(Sub_Pos)						;Whatever direction in moves in, we'll need the address
    call FindPixel						;in the mask
	ld de,Back_Mask
	add hl,de							;So we get that in hl,
    ld c,a								;and the offset in c
    pop af
	ld ix,Last_Move						;Set a pointer to the byte that needs to be changed for recording movement
	cp $ff								;(If no keys are pressed, check both)
	jr z,Decel_Check_All
	ld b,a
	and %00000110
	cp %00000110
	call z,Decel_Check_Horiz
	ld a,b
	and %00001001
	cp %00001001
	call z,Decel_Check_Vert
	ld a,b
    bit 0,a								;If one of the arrow keys has been pressed, move sub
    call z,Sub_Move_Down_Accel
    bit 2,a
    call z,Sub_Move_Right_Accel
    bit 1,a
    call z,Sub_Move_Left_Accel
    bit 3,a
    call z,Sub_Move_Up_Accel
	ld a,(New_Screen)					;Has it moved to a new screen?
	or a
	jp z,Draw_Sub						;If not, then just redraw the sub
	jp Draw_Screen						;Otherwise, redraw the background too
Decel_Check_All:
	push hl
	ld hl,(Sub_Vert_Delay_Coeff)
	ld de,Sub_Vert_Delay_1
	or a
	sbc hl,de
	pop hl
	jr nz,Decel_Check_All_Move
	push hl
	ld hl,(Sub_Horiz_Delay_Coeff)
	ld de,Sub_Horiz_Delay_1
	or a
	sbc hl,de
	pop hl
	jp z,Loop
Decel_Check_All_Move:
	call Decel
	ld a,(New_Screen)
	or a
	jp z,Draw_Sub
	jp Draw_Screen
Decel_Check_Vert:
	push hl
	ld hl,(Sub_Vert_Delay_Coeff)		;Is the sub standing still?
	ld de,Sub_Vert_Delay_1
	or a
	sbc hl,de
	pop hl
	ret z								;If not, it should be decelerating
	jr Decel
Decel_Check_Horiz:
	push hl
	ld hl,(Sub_Horiz_Delay_Coeff)
	ld de,Sub_Horiz_Delay_1
	or a
	sbc hl,de
	pop hl
	ret z
Decel:
	ld a,(Last_Move)					;so check on the direction of the last move
    bit 0,a								;and move accordingly
    call nz,Sub_Move_Down_Decel
    bit 2,a
    call nz,Sub_Move_Right_Decel
    bit 1,a
    call nz,Sub_Move_Left_Decel
    bit 3,a
    call nz,Sub_Move_Up_Decel
	ret


Contrast_Up:
	ld b,a
	ld hl,_contrast
	inc (hl)
	ld a,(hl)
	out (2),a
	call Title_Key_Delay
	ld a,b
	ret
Contrast_Down:
	ld b,a
	ld hl,_contrast
	dec (hl)
	ld a,(hl)
	out (2),a
	call Title_Key_Delay
	ld a,b
	ret


Redraw_All:
	xor a								;Show that the redraw is in progress
	ld (Redraw),a
	call Erase_Enemies					;then erase everything,
	call Erase_Dolphin
	call Erase_Ink
	call Erase_Enm_Proj
	ld a,(Sub_Flash)					;including the sub, if it's flashing,
	ld de,(Sub_Pos)
	or a
	ld a,Sub_Height
	call nz,Restore_Gray_Sprite
	call Draw_Objects					;then redraw everything
	call Draw_Enemies
	call Draw_Dolphin
	call Draw_Ink
	call Draw_Enm_Proj
	ld bc,(Sub_Pos)						;Finally, redraw the sub on top of everything else,
	ld hl,(Sub_Sprite)
	call Draw_Sprite
	ld a,(Draw_Status)					;draw the status display if needed,
	or a
	call nz,Draw_Status_Disp
	call Copy_From_Buffer				;and copy everything to the screen
	call Sub_Move_Test_Enemies			;test if the sub has hit any enemies,
	jp Sub_Move_Test_Dolphin			;and test if it's hit the dolphin


Show_Level_Info:
	xor a
	ld (New_Level),a
	ld de,Window_Text_Pos				;Position the cursor,
	ld (_penCol),de
	ld a,e
	ld (Start_Of_Line),a
	call Draw_Window					;then draw a window
	ld hl,New_Level_Text_1				;and display the current zone
	call Draw_Text
	ld hl,Curr_Zone
	call Draw_Text
	call _vnewline
	ld hl,New_Level_Text_2				;and sector
	call Draw_Text
	ld a,(Curr_Sector)
	ld l,a
	ld h,0
	call DisplayHL
	call Copy_From_Buffer
	call Wait_For_A_Key					;then wait for a keypress
	call Wait_For_No_Key
	call Erase_Window					;and erase the window
	jp Start_Show_Status				;Set to display amount of oxygen left


.end