;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Movement/collision detection code

;//////////////////////////////Movement w/acceleration, deceleration//////////////////////////////

Sub_Move_Up_Accel:
	set 3,(ix)
	res 0,(ix)
	push hl
	push bc
	call Sub_Move_Up
	jr Sub_Move_VertAccel
Sub_Move_Down_Accel:
	set 0,(ix)
	res 3,(ix)
	push hl
	push bc
	call Sub_Move_Down
Sub_Move_VertAccel:
	ld hl,(Sub_Vert_Delay_Coeff)		;Get the sub's current speed
	ld de,Sub_Vert_Delay_2
	push hl								;Is it at the upper limit?
	or a
	sbc hl,de
	pop hl
	jr c,Sub_Move_VertAccel_End			;If it is, then don't accelerate any more
	ld de,-Sub_Vert_Accel				;Otherwise, increase the speed
	add hl,de
	ld (Sub_Vert_Delay_Coeff),hl		;and store it
Sub_Move_VertAccel_End:
	ld (Sub_Delay_Coeff),hl
	pop bc
	pop hl
	ret


Sub_Move_Left_Accel:
	set 1,(ix)
	res 2,(ix)
	push hl
	push bc
	call Sub_Move_Left
	jr Sub_Move_HorizAccel
Sub_Move_Right_Accel:
	set 2,(ix)
	res 1,(ix)
	push hl
	push bc
	call Sub_Move_Right
Sub_Move_HorizAccel:
	ld hl,(Sub_Horiz_Delay_Coeff)
	ld de,Sub_Horiz_Delay_2
	push hl
	or a
	sbc hl,de
	pop hl
	jr c,Sub_Move_HorizAccel_End
	ld de,-Sub_Horiz_Accel
	add hl,de
	ld (Sub_Horiz_Delay_Coeff),hl
Sub_Move_HorizAccel_End:
	ld (Sub_Delay_Coeff),hl
	pop bc
	pop hl
	ret


Sub_Move_Up_Decel:
	push hl
	push bc
	call Sub_Move_Up
	jr Sub_Move_VertDecel
Sub_Move_Down_Decel:
	push hl
	push bc
	call Sub_Move_Down
Sub_Move_VertDecel:
	ld hl,(Sub_Vert_Delay_Coeff)		;Decrease the sub's speed
	ld de,Sub_Vert_Accel
	add hl,de
	ld (Sub_Vert_Delay_Coeff),hl		;and store it
	ld (Sub_Delay_Coeff),hl
	ld de,Sub_Vert_Delay_1
	or a								;Is it at the lower limit?
	sbc hl,de
	call z,Sub_Move_VertDecel_End		;If it is, then don't decelerate any more
	pop bc
	pop hl
	ret
Sub_Move_VertDecel_End:
	res 0,(ix)
	res 3,(ix)
	ret


Sub_Move_Left_Decel:
	push hl
	push bc
	call Sub_Move_Left
	jr Sub_Move_HorizDecel
Sub_Move_Right_Decel:
	push hl
	push bc
	call Sub_Move_Right
Sub_Move_HorizDecel:
	ld hl,(Sub_Horiz_Delay_Coeff)
	ld de,Sub_Horiz_Accel
	add hl,de
	ld (Sub_Horiz_Delay_Coeff),hl
	ld (Sub_Delay_Coeff),hl
	ld de,Sub_Horiz_Delay_1
	or a
	sbc hl,de
	call z,Sub_Move_HorizDecel_End
	pop bc
	pop hl
	ret
Sub_Move_HorizDecel_End:
	res 1,(ix)
	res 2,(ix)
	ret

;//////////////////////////////Vertical movement//////////////////////////////

Sub_Move_Up:
	ld (Sub_Move_Vert_End_Save),a
	ld de,Screen_Width					;Get the address of the top of the sub in mask
	or a
	sbc hl,de
	call Check_Horiz_Empty				;and check if the area above the sub is empty
	jr c,Sub_Move_Vert_Ret				;If not, go back to key loop
	ld hl,Sub_Pos_Y						;If the sub is free to move, move it up now
	dec (hl)							;Move up 1 pixel
	call z,Go_To_Screen_On_Top			;If it's at top of screen, go to screen on top
	jr Sub_Move_Vert_End

Sub_Move_Down:
	ld (Sub_Move_Vert_End_Save),a
	ld de,Screen_Width*Sub_Height		;Get address of bottom of sub in mask
	add hl,de
	call Check_Horiz_Empty				;and check if it's empty
	jr c,Sub_Move_Vert_Ret				;If not, go back to key loop
	ld hl,Sub_Pos_Y						;If the sub is free to move, move it down now
	inc (hl)							;Move down 1 pixel
	ld a,(hl)
	cp Screen_Height_P-Sub_Height
	call z,Go_To_Screen_On_Bottom		;If it's at bottom of screen, go to screen on bottom
Sub_Move_Vert_End:
	call Sub_Move_Test_Objects
Sub_Move_Vert_Ret:
Sub_Move_Vert_End_Save	equ	$+1
	ld a,0
	ret

Check_Horiz_Empty:						;Check 2-byte wide area at (hl) - if not empty, c is set
	ld a,(hl)
	or a
	jr nz,Check_Horiz_Not_Empty
	inc hl
	ld a,(hl)
	dec hl
	or a
	jr nz,Check_Horiz_Not_Empty
	ret
Check_Horiz_Not_Empty:
	scf
	ret

;//////////////////////////////Horizontal movement//////////////////////////////

Sub_Move_Left:
	ld (Sub_Move_Horiz_End_Save),a
	call Check_Vert_Empty				;Check if area on left side of sub is empty
	jr c,Sub_Move_Horiz_Ret				;If not, go back to key loop
	ld hl,Sub_Pos_X						;If it is empty, move left now
	dec (hl)							;Move left 1 pixel
	call z,Go_To_Screen_On_Left			;If at left edge of screen, go to screen on left
Sub_Move_Left_Draw:
	ld hl,sub_left						;Set required vars to indicate direction
	ld a,1
	jr Sub_Move_Horiz_End

Sub_Move_Right:
	ld (Sub_Move_Horiz_End_Save),a
	inc hl
	ld a,c
	cp $80
	jr z,$+3
	inc hl
	call Check_Vert_Empty				;Check if area on right side of sub is empty
	jr c,Sub_Move_Horiz_Ret				;If not, go back to key loop
	ld hl,Sub_Pos_X						;If it is empty, move right now
	inc (hl)							;Move right 1 pixel
	ld a,(hl)
	cp Screen_Width_P-Sub_Width
	call z,Go_To_Screen_On_Right		;If at right edge of screen, go to screen on right
Sub_Move_Right_Draw:
	ld hl,sub_right						;Set required vars to indicate direction
	xor a
Sub_Move_Horiz_End:
	ld (Sub_Dir),a
	ld a,(Sub_Flash)					;Before setting the sprite to use for the sub, check if it's supposed to be flashing
	or a
	jr z,Sub_Move_Horiz_Normal
Sub_Move_Horiz_Flash:
	ld (Sub_Sprite_Save),hl
	jr $+5
Sub_Move_Horiz_Normal:
	ld (Sub_Sprite),hl
	call Sub_Move_Test_Objects
Sub_Move_Horiz_Ret:
	ld de,(Sub_Pos)						;Find the sub's position in the current map
	ld a,d
	and %01111000
	add a,a
	ld d,a
	ld a,e
	add a,Sub_Width
	and %11111000
	rra
	rra
	rra
	add a,d
	ld hl,Curr_Back2
	add a,l
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	ld a,(hl)							;and check the tile it's touching
	cp Station_Door_Tile1				;If it's touching the door of the station,
	jp z,Next_Level						;then we're done with this level!!!!!
	cp Station_Door_Tile2
	jp z,Next_Level
Sub_Move_Horiz_End_Save	equ	$+1
	ld a,0
	ret

Check_Vert_Empty:						;Check 10-byte column at (hl) - if not empty, c is set
	push hl
	ld b,Sub_Height
	ld de,Screen_Width
Check_Vert_Empty_Loop:
	ld a,(hl)
	and c
	jr nz,Check_Vert_Not_Empty
	add hl,de
	djnz Check_Vert_Empty_Loop
	xor a
	pop hl
	ret
Check_Vert_Not_Empty:
	scf
	pop hl
	ret

;//////////////////////////////Collisions//////////////////////////////

Sub_Move_Test_Enemies:
	ld a,(Sub_Flash)						;First check if sub is temporarily "invincible"
	or a
	ret nz									;If so, skip this code
	ld bc,Sub_Height*256+Sub_Width
	ld de,(Sub_Pos)
	ld hl,(Sub_Sprite)
	call Test_Hit_Enemy						;Otherwise, check for collisions
	ret nc
	jp Dec_Lives							;If it's hit an enemy, remove a unit of oxygen


Sub_Move_Test_Dolphin:
	ld hl,(Curr_Dolphin_Status)				;Make sure that the dolphin exists in this screen before wasting time on this crap
	bit bitDExists,(hl)
	ret z
	bit bitDHasTank,(hl)					;(and make sure it hasn't already given up its oxygen tank)
	ret z
	ld de,(Dolphin_Pos)						;Get the dolphin's position
	ld bc,(Sub_Pos)							;and the sub's position
	ld a,c									;then compare them
	add a,Sub_Width
	cp e
	ret c
	ld a,e
	add a,Dolphin_Width
	cp c
	ret c
	ld a,b
	add a,Sub_Height
	cp d
	ret c
	ld a,d
	add a,Dolphin_Height
	cp b
	ret c									;If they're touching,
	res bitDHasTank,(hl)					;then remove the dolphin's oxygen tank
	ld a,(Oxygen)							;and add oxygen to the sub
	ld b,a
	and %11111100							;Ah, but first we have to check if the sub's on a full tank or not!
	cp b
	jr nz,Sub_Move_Test_Dolphin_FillTank
Sub_Move_Test_Dolphin_AddTank:				;If it is, then add a new tank of oxygen!
	ld a,b
	add a,4
	ld (Oxygen),a
	jp Start_Show_Status					;(and show the new amount)
Sub_Move_Test_Dolphin_FillTank:
	add a,4									;If the current tank wasn't full, then refill it
	ld (Oxygen),a
	jp Start_Show_Status


Sub_Move_Test_Objects:
	ld de,(Sub_Pos)
	ld hl,(Curr_Fragments)					;Start with the first fragment
	ld b,Max_Fragment_Num					;Init loop counter
Sub_Move_Test_Objects_Loop1:
	push bc
	ld a,(hl)
	inc hl
	ld c,(hl)								;Get the current object's position
	inc hl
	ld b,(hl)
	inc hl
	or a									;Does the object even exist?
	jr z,Sub_Move_Test_Objects_Loop1_End	;If not, skip this one
	call Sub_Move_Test_Objects_Test			;Otherwise, test its position against the sub's position
	jr c,Sub_Move_Test_Objects_Loop1_End	;If they're not touching, move on to the next object
	push hl
	push de
	dec hl
	dec hl
	dec hl
	ld (hl),0								;If they are, erase the object from the tables
	ld d,b
	ld e,c
	ld a,Object_Height
	call Restore_Gray_Sprite				;and the screen,
	ld a,1
	ld (Redraw),a							;show that a redraw is needed,
	ld hl,Num_Objects						;and decrement the number of remaining objects
	dec (hl)
	pop de
	pop hl
Sub_Move_Test_Objects_Loop1_End:
	pop bc
	djnz Sub_Move_Test_Objects_Loop1

	ld b,Max_Canister_Num					;Check the data canisters too
Sub_Move_Test_Objects_Loop2:
	push bc
	ld a,(hl)
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	or a
	jr z,Sub_Move_Test_Objects_Loop2_End
	push de
	ld de,(Sub_Pos)
	call Sub_Move_Test_Objects_Test
	pop de
	jr c,Sub_Move_Test_Objects_Loop2_End
	push hl
	dec hl
	dec hl
	dec hl
	dec hl
	dec hl
	ld (hl),0
	push de
	ld d,b
	ld e,c
	ld a,Object_Height
	call Restore_Gray_Sprite
	ld a,1
	ld (Redraw),a
	ld hl,Num_Objects
	dec (hl)
	pop de									;Ah, but this time, if we've picked up a data canister,
	call Draw_Canister_Text					;we've got to show the contents!
	pop hl
Sub_Move_Test_Objects_Loop2_End:
	pop bc
	djnz Sub_Move_Test_Objects_Loop2

	ret

Sub_Move_Test_Objects_Test:
	ld a,c
	add a,Object_Width
	cp e
	ret c
	ld a,e
	add a,Sub_Width
	cp c
	ret c
	ld a,b
	add a,Object_Height
	cp d
	ret c
	ld a,d
	add a,Sub_Height
	cp b
	ret

;//////////////////////////////Screen load//////////////////////////////

Go_To_Screen_On_Top:
	ld a,Screen_Height_P-Sub_Height-2
	ld (Sub_Pos_Y),a
	ld a,(Curr_Dir_Table)
	jr Load_New_Screen
Go_To_Screen_On_Bottom:
	ld a,1
	ld (Sub_Pos_Y),a
	ld a,(Curr_Dir_Table+1)
	jr Load_New_Screen
Go_To_Screen_On_Left:
	ld a,Screen_Width_P-Sub_Width-2
	ld (Sub_Pos_X),a
	ld a,(Curr_Dir_Table+2)
	jr Load_New_Screen
Go_To_Screen_On_Right:
	ld a,1
	ld (Sub_Pos_X),a
	ld a,(Curr_Dir_Table+3)
Load_New_Screen:
;PATCH
	or a							;(if the screen doesn't exist, then there's a bug in the
	jr z,Load_New_Screen_Ret		;wall collision detection, so don't load the new screen)
;END PATCH
	push af
	dec a
	ld l,a							;Re-calculate the object table pointers
	ld h,Fragment_Table_Len+Canister_Table_Len+1
	call _htimesl
	ld de,Obj_Table
	add hl,de
	ld (Curr_Fragments),hl
	ld de,Fragment_Table_Len
	add hl,de
	ld (Curr_Canisters),hl
	ld de,Canister_Table_Len
	add hl,de
	ld (Curr_Dolphin_Status),hl
	pop af
	ld b,a							;b=index number of screen to load
	ld a,(Level_Start_Addr)			;Get the abs address of the level string in memory,
	ld hl,(Level_Start_Addr+1)
	ld de,Screen_Len
	dec b
	jr z,Load_New_Screen_Skip_Mult
Load_New_Screen_Mult_Loop:			;then add (scrn_num-1)*screen_len to find the location
	add hl,de						;of the screen data in the level string
	adc a,0
	djnz Load_New_Screen_Mult_Loop
Load_New_Screen_Skip_Mult:
	call _SET_ABS_SRC_ADDR			;and copy it from there
	ld a,$01
	ld hl,Copy_Addr-$8000			;to the 'current screen' location on RAM page 1
	call _SET_ABS_DEST_ADDR
	xor a
	ld hl,Screen_Len
	call _SET_MM_NUM_BYTES
	di								;(stop interrupts so the calc won't crash while other RAM
	call _MM_LDIR					;pages are loaded during the abs copy)
	ei								;Afterwards, restart the grayscale ISR,
	call Init_Enemies				;initialize enemies for this screen,
	ld a,1							;and signal that a redraw of the background is needed
	ld (New_Screen),a
	ret
;MORE PATCH
Load_New_Screen_Ret:
	ld hl,(Old_Sub_Pos)
	ld (Sub_Pos),hl
	ret
;END PATCH

.end