;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Data canister/capsule fragment code


;Data table format

;=====================(4 individual object tables)
;(0)-Status of object
;(1-2)-Position
;(3-4)-Offset of text from Level_Start_Addr [data canisters only]

;Status byte format

;(bit)
;0-object exists
;1-reset if on frame 1, set if on frame 2

;//////////////////////////////Initialization//////////////////////////////

Init_Objects:
	ld a,Fragment_Anim_Delay			;Init delays
	ld (Fragment_Anim_Counter),a
	ld a,Canister_Anim_Delay
	ld (Canister_Anim_Counter),a
	ret

;//////////////////////////////Animate//////////////////////////////

Anim_Objects:
	ld a,(Object_Anim_Enable)			;(First, make sure object animation is enabled)
	or a
	ret z
	ld hl,Fragment_Anim_Counter			;Decrement the first counter
	dec (hl)
	call z,Anim_Objects_Fragment		;If it's 0, animate the capsule fragments
	ld hl,Canister_Anim_Counter			;Do the same for the data canisters
	dec (hl)
	jr z,Anim_Objects_Canister
	ret

Anim_Objects_Fragment:					;When animating the capsule fragments,
	ld (hl),Fragment_Anim_Delay			;first reset the counter
	ld hl,(Curr_Fragments)				;then check each individual piece
	bit 0,(hl)							;Does the first one exist?
	jr z,Anim_Objects_Fragment_Skip
	ld a,1
	ld (Redraw),a
	bit 1,(hl)							;If so, flip its frame bit
	jr z,$+6
	res 1,(hl)
	jr Anim_Objects_Fragment_Skip
	set 1,(hl)
Anim_Objects_Fragment_Skip:
	inc hl
	inc hl
	inc hl
	bit 0,(hl)							;(same for the second one)
	ret z
	ld a,1
	ld (Redraw),a
	bit 1,(hl)
	jr z,$+5
	res 1,(hl)
	ret
	set 1,(hl)
	ret

Anim_Objects_Canister:
	ld (hl),Canister_Anim_Delay
	ld hl,(Curr_Canisters)
	bit 0,(hl)							;Does the first one exist?
	jr z,Anim_Objects_Canister_Skip
	ld a,1
	ld (Redraw),a
	bit 1,(hl)							;If so, flip its frame bit
	jr z,$+6
	res 1,(hl)
	jr Anim_Objects_Canister_Skip
	set 1,(hl)
Anim_Objects_Canister_Skip:
	ld de,5
	add hl,de
	bit 0,(hl)							;(same for the second one)
	ret z
	ld a,1
	ld (Redraw),a
	bit 1,(hl)
	jr z,$+5
	res 1,(hl)
	ret
	set 1,(hl)
	ret

;//////////////////////////////Draw//////////////////////////////

Draw_Objects:
	ld hl,(Curr_Fragments)
	ld a,(hl)							;Get the first object's status
	inc hl
	ld c,(hl)							;and position
	inc hl
	ld b,(hl)
	inc hl
	or a								;Wait - does it even exist?
	jr z,Draw_Objects_Skip1
	push hl
	ld hl,fragment1
	bit 1,a								;If it does, check which frame it's on
	jr z,$+5							;Find the right sprite from that...
	ld hl,fragment2
	call Draw_Sprite					;and draw it
	pop hl
Draw_Objects_Skip1:
	ld a,(hl)							;Now for the rest of them...
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	inc hl
	or a
	jr z,Draw_Objects_Skip2
	push hl
	ld hl,fragment1
	bit 1,a
	jr z,$+5
	ld hl,fragment2
	call Draw_Sprite
	pop hl
Draw_Objects_Skip2:
	ld a,(hl)
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	inc hl
	inc hl
	inc hl
	or a
	jr z,Draw_Objects_Skip3
	push hl
	ld hl,canister1
	bit 1,a
	jr z,$+5
	ld hl,canister2
	call Draw_Sprite
	pop hl
Draw_Objects_Skip3:
	ld a,(hl)
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	inc hl
	inc hl
	inc hl
	or a
	ret z
	push hl
	ld hl,canister1
	bit 1,a
	jr z,$+5
	ld hl,canister2
	call Draw_Sprite
	pop hl
	ret


Draw_Canister_Text:			;text offset in de
	ld a,(Level_Start_Addr)
	ld hl,(Level_Start_Addr+1)
	add hl,de
	adc a,0									;Find the abs address of its text,
	call _abs_to_page_asic
	ld de,-$4000
	add hl,de
	set 6,a
	di
	out (5),a								;and load the correct mem pages
	inc a
	out (6),a
	ld de,Temp_Canister_Text
Draw_Canister_Text_Copy_Loop1:
	ld a,(hl)								;Find the number of lines to draw
	ld (de),a
	inc hl
	inc de
	or a									;If it's 0, then it's the end of this data canister's text
	jr z,Draw_Canister_Text_Draw
	ld b,a									;Otherwise, init the loop counter,						
Draw_Canister_Text_Copy_Loop2:
	ld a,(hl)								;and copy each line
	ld (de),a
	inc hl
	inc de
	or a
	jr nz,Draw_Canister_Text_Copy_Loop2
	djnz Draw_Canister_Text_Copy_Loop2
	jr Draw_Canister_Text_Copy_Loop1
Draw_Canister_Text_Draw:
	ld a,$0D								;When it's all been copied to page 0,
	out (5),a								;restore the pages
	ld a,$41
	out (6),a
	ld hl,Temp_Canister_Text				;then draw the text
	ei
Draw_Canister_Text_Draw_Loop:
	ld a,(hl)
	or a
	jp z,Erase_Window
	inc hl
	push hl
	call Draw_Window
	pop hl
	ld b,a						
	call Draw_Window_Text
	push hl
	call Copy_From_Buffer					;When each window of text is done, show it all on the screen,
	pop hl
	call Wait_For_A_Key						;wait for a keypress,
	call Wait_For_No_Key
	jr Draw_Canister_Text_Draw_Loop			;and go back for the next screen of text

.end