;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Routines

Get_Keypress:				;Select key group in a, get key in a
    out (1),a
    nop
    nop
    in a,(1)
    ret


Wait_For_No_Key:			;Returns when no keys pressed
    ld a,k_g1
	call Get_Keypress
	cp $ff
	jr nz,Wait_For_No_Key
	ret


Wait_For_A_Key:				;Returns when key pressed
	ld a,k_g1
	call Get_Keypress
	cp $ff
	jr z,Wait_For_A_Key
	ret


Draw_Window:				;Draws window in screen buffers
	ld hl,Gfx_Buffer_1+Window_Offset
	ld b,Window_Width
Draw_Window_Loop1:
	ld (hl),0
	inc hl
	djnz Draw_Window_Loop1
	ld de,Screen_Width-Window_Width
	add hl,de
	inc de
	ld b,Window_Height-2
Draw_Window_Loop2:
	ld (hl),$7f
	inc hl
	ld c,Window_Width-2
Draw_Window_Loop2_Inner:
	ld (hl),$ff
	inc hl
	dec c
	jr nz,Draw_Window_Loop2_Inner
	ld (hl),$fe
	add hl,de
	djnz Draw_Window_Loop2
	ld b,Window_Width
Draw_Window_Loop3:
	ld (hl),0
	inc hl
	djnz Draw_Window_Loop3
	ld hl,Gfx_Buffer_2+Window_Offset
	ld de,Screen_Width-Window_Width
	ld b,Window_Height
	ld c,Window_Width
Draw_Window_Loop4:
	ld (hl),$ff
	inc hl
	dec c
	jr nz,Draw_Window_Loop4
	add hl,de
	ld c,Window_Width
	djnz Draw_Window_Loop4
	ret


Draw_Window_Text:			;Draws string at (hl) with b lines in window
	ld de,Window_Text_Pos				;Set the cursor
	ld (_penCol),de
	ld a,e
	ld (Start_Of_Line),a
Draw_Window_Text_Loop:
	push bc
	call Draw_Text						;then draw
	push hl
	call _vnewline
	pop hl
	pop bc
	djnz Draw_Window_Text_Loop
	ret


Erase_Window:							;Erase window from background buffers
	ld de,Window_Pos
	ld a,Window_Height
	call Restore_Gray_Sprite
	ld de,Window_Pos+24
	ld a,Window_Height
	call Restore_Gray_Sprite
	ld de,Window_Pos+48
	ld a,Window_Height
	call Restore_Gray_Sprite
	ld de,Window_Pos+72
	ld a,Window_Height
	jp Restore_Gray_Sprite


Draw_Text:								;Draw string at (hl) in both buffers
	ld de,(_penCol)
	push de
	ld a,Gfx_Buffer_1/256
	ld (fvputc_vidplane),a				;Draw the text in the first buffer,
	push hl
	call fvputs
	pop hl
	pop de
	ld (_penCol),de
	ld a,Gfx_Buffer_2/256
	ld (fvputc_vidplane),a				;then the second buffer
	jp fvputs


Mult_AB:					;HL=A*B
	ld h,0
	ld l,a
	dec b
	ret z
	ld d,h
	ld e,a
Mult_AB_Loop:
	add hl,de
	djnz Mult_AB_Loop
	ret


Mirror_Sprites:				;Mirrors sprites from Sprites_To_Mirror to Sprites_To_Mirror_End, copies to Mirrored_Sprites_Dest
	ld hl,Sprites_To_Mirror
	ld de,Mirrored_Sprite_Dest
Mirror_Sprites_Loop:
	ld bc,Sprites_To_Mirror_End			;Have all the sprites been mirrored yet?
	or a
	push hl
	sbc hl,bc
	pop hl
	ret z								;If they have, it's done
	ld a,(hl)							;If not, look at the next sprite
	ld (de),a
	ld (Mirror_Sprites_Cols),a
	dec a
	ld c,a
	inc hl
	inc de
	ld b,(hl)
	ld a,b
	ld (de),a
	add a,a
	add a,b
	ld (Mirror_Sprites_Rows),a
	ld b,a
	add a,a
	ld (Mirror_Sprites_2Cols),a
	ld a,c
	inc hl
	inc de
	push hl
	push de
	push hl
	call Mult_AB						;Find the address of the last column
	ld b,h
	ld c,l
	pop hl
	add hl,bc
	pop de
Mirror_Sprites_Cols	equ	$+1
	ld b,0
Mirror_Sprites_Inner_Loop:
	push bc
Mirror_Sprites_Rows	equ	$+1
	ld b,0
Mirror_Sprites_Col_Loop:
	ld a,(hl)							;Read the byte in the current column,
	push hl
	push de
	push bc
	call byteGetMirror					;reverse it,
	pop bc
	pop de
	pop hl
	ld (de),a							;and write it to the new sprite
	inc hl								;then move on to the next row
	inc de
	djnz Mirror_Sprites_Col_Loop
Mirror_Sprites_2Cols	equ	$+1
	ld c,0								;When done with that column,
	ld b,0
	or a
	sbc hl,bc							;go to the previous column in the input sprite
	pop bc
	djnz Mirror_Sprites_Inner_Loop		;When all the columns in that sprite are done,
	ld a,(Mirror_Sprites_Cols)			;move on to the next sprite
	ld b,a
	ld a,(Mirror_Sprites_Rows)
	push de
	call Mult_AB
	pop de
	ld b,h
	ld c,l
	pop hl
	add hl,bc
	jr Mirror_Sprites_Loop


Test_Hit_Enemy:				;Position in de, size in bc, and sprite in hl, c set if touching enemy
	ld (Test_Hit_Enemy_Mask_sprite2),hl
	ld (Test_Hit_Enemy_Pos),de
	ld a,b
	ld (Test_Hit_Enemy_H),a
	ld a,c
	ld (Test_Hit_Enemy_W),a
	ld a,(Enemy_Num)						;Check the number of enemies
	or a
	ret z									;If there aren't any, we don't need to worry about collisions
	ld b,a
	ld hl,Enemy_1_Pos
	ld (Enemy_Pos_Ptr),hl
	ld hl,Enemy_1_Type
	ld (Enemy_Data_Ptr),hl
Test_Hit_Enemy_Loop:
	push bc
	ld hl,(Enemy_Data_Ptr)
	bit bitInCloud,(hl)						;If the enemy is in an ink cloud, skip it
	jp nz,Test_Hit_Enemy_End
	ld a,(hl)								;find sprite and size of enemy
	call Find_Enemy_Sprite
	ld (Test_Hit_Enemy_Mask_sprite1),hl
	ld a,(hl)								;and get its size from that
	rla
	rla
	rla
	ld b,a
	inc hl
	ld c,(hl)
	ld hl,(Enemy_Pos_Ptr)					;Get position of current enemy
	call _ldHLind
Test_Hit_Enemy_Pos	equ	$+1
	ld de,0									;and sub
;hl=enemy pos, de=sub pos, bc=enemy size
	ld a,h									;Test w/positions
	add a,c
	cp d
	jp c,Test_Hit_Enemy_End
	ld a,d
Test_Hit_Enemy_H	equ	$+1
	add a,0
	cp h
	jp c,Test_Hit_Enemy_End
	ld a,l
	add a,b
	cp e
	jp c,Test_Hit_Enemy_End
	ld a,e
Test_Hit_Enemy_W	equ	$+1
	add a,0
	cp l
	jp c,Test_Hit_Enemy_End
;ONE BIG HACK - Newbies, do not use as an example!
Test_Hit_Enemy_Mask:
	push hl									;If the sprites are touching, then we've got to check
Test_Hit_Enemy_Mask_sprite1	equ	$+1			;the masks to see if there's really a collision - NOOOOOOOOOOOOOOO!!!!!!!!!!!
	ld hl,0
	ld a,(hl)								;Get height and width of first sprite and save them
	ld (Test_Hit_Enemy_Mask_w1),a
	ld (Test_Hit_Enemy_Mask_w1_save),a
	inc hl
	ld a,(hl)
	ld (Test_Hit_Enemy_Mask_h1),a
	inc hl
	add a,a
	ld c,a
	ld b,0
	add hl,bc
	ld (Test_Hit_Enemy_Mask_mask1),hl		;then find the address of the mask of the first column and save that too
	ld (Test_Hit_Enemy_Mask_spr1_save),hl
	ld a,e
	ld (Test_Hit_Enemy_Mask_x1),a
Test_Hit_Enemy_Mask_sprite2	equ	$+1
	ld hl,0									;Do the same with the second sprite
	ld a,(hl)
	ld (Test_Hit_Enemy_Mask_w2),a
	inc hl
	ld a,(hl)
	ld (Test_Hit_Enemy_Mask_h2),a
	inc hl
	add a,a
	ld c,a
	ld b,0
	add hl,bc
	ld (Test_Hit_Enemy_Mask_mask2),hl
	pop hl
	ld b,0								;compare sprite y positions
	ld a,h
	cp d
	ld a,(Test_Hit_Enemy_Mask_h2)
	ld c,a
	ld a,(Test_Hit_Enemy_Mask_h1)
	jr c,Test_Hit_Enemy_Mask_endtesty
	ex de,hl
	inc b								;(0 - sprite 1 (hl) is top, sprite 2 (de) is bottom, 1 - sprite 2 (de) is top, sprite 1 (hl) is bottom)
	ld a,(Test_Hit_Enemy_Mask_h1)
	ld c,a
	ld a,(Test_Hit_Enemy_Mask_h2)
Test_Hit_Enemy_Mask_endtesty:			;hl=top sprite pos, de=bottom sprite pos
	ld (Test_Hit_Enemy_Mask_ht),a
	ld a,c
	ld (Test_Hit_Enemy_Mask_hb),a
	push bc
	ld a,d
	sub h								;a=downwards offset for bottom sprite
	ld c,a
Test_Hit_Enemy_Mask_hb	equ	$+1			;calculate number of rows to test - Hb if Ob+Hb=<Ht, Ht-Ob otherwise
	ld b,0
	add a,b								;a=Ob+Hb
Test_Hit_Enemy_Mask_ht	equ	$+1
	ld b,0								;b=Ht
	cp b
	jr c,Test_Hit_Enemy_UseHB
	ld a,b								;option 1: # rows = Ht-Ob
	sub c
	jr Test_Hit_Enemy_SetNumRows
Test_Hit_Enemy_UseHB:
	ld a,(Test_Hit_Enemy_Mask_hb)		;option 2: # rows = Hb
Test_Hit_Enemy_SetNumRows:
	ld (Test_Hit_Enemy_Mask_rows),a				;set the number of rows to test
	ld a,c
	pop bc
	djnz Test_Hit_Enemy_Mask_setoffset1			;if there's a difference in height, find which sprite to offset
Test_Hit_Enemy_Mask_setoffset2:
	ld b,a
	xor a
	jr Test_Hit_Enemy_Mask_endoffset
Test_Hit_Enemy_Mask_setoffset1:
	ld b,0
Test_Hit_Enemy_Mask_endoffset:					;offset the correct sprite...
	ld (Test_Hit_Enemy_Mask_offset1),a
	ld a,b
	ld (Test_Hit_Enemy_Mask_offset2),a
Test_Hit_Enemy_Mask_Full_Loop:			;(loop to compare columns of sprite 1 and sprite 2)
	push hl
	push de
	xor a
	ld (Test_Hit_Enemy_Mask_lr),a
	ld a,l								;compare sprite x positions
	cp e
	jr c,Test_Hit_Enemy_Mask_endtestx
	ex de,hl
	ld a,$eb ;ex de,hl					;(0 - sprite 1 (hl) is left, sprite 2 (de) is right, 1 - sprite 2 (de) is left, sprite 1 (hl) is right)
	ld (Test_Hit_Enemy_Mask_lr),a
Test_Hit_Enemy_Mask_endtestx:			;hl=left sprite pos, de=right sprite pos
	ld a,e
	sub l								;a=amount to shift left sprite left
	ld (Test_Hit_Enemy_Mask_sl),a
Test_Hit_Enemy_Mask_mask2	equ	$+1
	ld hl,0
Test_Hit_Enemy_Mask_offset2	equ	$+1
	ld c,0
	ld b,0
	add hl,bc
	ex de,hl
Test_Hit_Enemy_Mask_mask1	equ	$+1
	ld hl,0
Test_Hit_Enemy_Mask_offset1	equ	$+1
	ld c,0
	ld b,0
	add hl,bc							;hl=sprite 1, de=sprite 2
Test_Hit_Enemy_Mask_lr	equ	$
	nop									;switch them if necessary
Test_Hit_Enemy_Mask_rows	equ	$+1
	ld b,8
	ld a,b								;if there aren't any overlapping rows,
	or a
	jr z,Test_Hit_Enemy_Next_Col		;then the sprites must not be overlapping!
Test_Hit_Enemy_Mask_loop:
	push bc								;otherwise, save the number of rows left to test
	ld c,(hl)							;then get the mask byte from the left sprite
Test_Hit_Enemy_Mask_sl	equ	$+1
	ld b,0								;and shift it the correct number of times
	ld a,b
	or a
	jr z,Test_Hit_Enemy_Mask_noshift
Test_Hit_Enemy_Mask_shiftloop:
	sla c
	djnz Test_Hit_Enemy_Mask_shiftloop
Test_Hit_Enemy_Mask_noshift:
	ld a,(de)							;then overlay it with the mask byte from the right-hand sprite
	and c								;the big test...
	pop bc
	jr nz,Test_Hit_Enemy_Y				;if there's an overlap, there's a collision - set the var to 1
	inc hl
	inc de
	djnz Test_Hit_Enemy_Mask_loop		;otherwise, move on to the next row
Test_Hit_Enemy_Next_Col:
	pop de
	pop hl
Test_Hit_Enemy_Mask_w1	equ	$+1
	ld a,0								;How many columns are left to go?
	dec a
	jr z,Test_Hit_Enemy_Mask_1col		;If only 1, we're done, so skip the rest of this crap
	ld (Test_Hit_Enemy_Mask_w1),a		;Otherwise, dec the col counter
	push hl
	ld hl,(Test_Hit_Enemy_Mask_mask1)	;then update the mask pointer to point to the mask of the next column of the sprite,
Test_Hit_Enemy_Mask_h1	equ	$+1
	ld a,0
	ld c,a
	add a,a
	add a,c
	ld c,a
	ld b,0
	add hl,bc
	ld (Test_Hit_Enemy_Mask_mask1),hl
	pop hl
	ld a,e								;update the x position,
	add a,8
	ld e,a
	jr Test_Hit_Enemy_Mask_Full_Loop	;and loop again
Test_Hit_Enemy_Mask_1col:				;When all the columns of sprite 1 have been tested,
Test_Hit_Enemy_Mask_w2	equ	$+1
	ld a,0								;see how many columns of sprite 2 are left to test
	dec a
	jr z,Test_Hit_Enemy_End				;If there aren't any, then FINALLY move on to the next enemy - no collisions here!
	ld (Test_Hit_Enemy_Mask_w2),a		;Otherwise, *sigh* update the column counter
Test_Hit_Enemy_Mask_w1_save	equ	$+1
	ld a,0								;and reset the column counter and pointer for sprite 1
	ld (Test_Hit_Enemy_Mask_w1),a		;(because we need to test ALL of it against the next column of sprite 2!)
Test_Hit_Enemy_Mask_spr1_save	equ	$+1
	ld bc,0
	ld (Test_Hit_Enemy_Mask_mask1),bc
	push hl
	ld hl,(Test_Hit_Enemy_Mask_mask2)	;then, just like before, update all the pointers and the position and stuff,
Test_Hit_Enemy_Mask_h2	equ	$+1
	ld a,0
	ld c,a
	add a,a
	add a,c
	ld c,a
	ld b,0
	add hl,bc
	ld (Test_Hit_Enemy_Mask_mask2),hl
	pop hl
	ld a,l
	add a,8
	ld l,a
Test_Hit_Enemy_Mask_x1	equ	$+1
	ld e,0
	jp Test_Hit_Enemy_Mask_Full_Loop	;and loop again
Test_Hit_Enemy_Y:
	pop hl								;If collision, get rid of positions and other junk
	pop hl								;saved on stack,
	pop bc
	scf									;set the carry flag to indicate a collision,
	ret									;and return
Test_Hit_Enemy_End:						;When done testing current enemy, update pointers
	pop bc
	ld hl,(Enemy_Pos_Ptr)
	inc hl
	inc hl
	ld (Enemy_Pos_Ptr),hl
	ld hl,(Enemy_Data_Ptr)
	ld de,Enemy_Data_Len
	add hl,de
	ld (Enemy_Data_Ptr),hl
	dec b
	jp nz,Test_Hit_Enemy_Loop			;and check next enemy
	xor a								;Return with c reset if all enemies have been checked and there aren't any collisions
	ret


Test_Hit_Wall:				;Position in de, c set if touching vertical wall
    call FindPixel						;Again just a hack, only here until I can motivate
	ld de,Back_Mask						;myself to write something better
	add hl,de
    ld c,a
	ld de,Screen_Width
	ld b,8
Test_Hit_Wall_Loop:
	ld a,(hl)
	and c
	jr nz,Test_Hit_Wall_Y
	add hl,de
	djnz Test_Hit_Wall_Loop
	xor a
	ret
Test_Hit_Wall_Y:
	scf
	ret


Test_Hit_Sub:					;c set if projectile is touching sub
	ld hl,(Proj_Sprite)
	ld a,(hl)
	rla
	rla
	rla
	ld b,a
	inc hl
	ld c,(hl)
	ld hl,(Proj_Pos)
	ld de,(Sub_Pos)
	ld a,h
	add a,c
	cp d
	jp c,Test_Hit_Sub_N
	ld a,d
	add a,Sub_Height
	cp h
	jp c,Test_Hit_Sub_N
	ld a,l
	add a,b
	cp e
	jp c,Test_Hit_Sub_N
	ld a,e
	add a,Sub_Width
	cp l
	jp c,Test_Hit_Sub_N
Test_Hit_Sub_Y:
	scf
	ret
Test_Hit_Sub_N:
	xor a
	ret


Find_Enemy_Sprite:				;enemy type byte in a, returns sprite in hl
	bit bitAnimState,a					;Is it on the second frame of animation?
	jr nz,Find_Enemy_Sprite_Alt			;If so, use alternate sprite
	bit bitMoveDir,a					;Is it moving right instead?
	jr nz,Find_Enemy_Sprite_Right
Find_Enemy_Sprite_Left:
	ld hl,Enemy_Sprite_Table_Left
Find_Enemy_Sprite_End:
	and $0f
	add a,a
	ld e,a
	ld d,0
	add hl,de
	jp _ldHLind							;then find the address of the sprite from the table
Find_Enemy_Sprite_Alt:
	bit bitMoveDir,a
	jr nz,Find_Enemy_Sprite_Alt_Right
Find_Enemy_Sprite_Alt_Left:
	ld hl,Enemy_Sprite_Table_Alt_Left
	jr Find_Enemy_Sprite_End
Find_Enemy_Sprite_Alt_Right:
	ld hl,Enemy_Sprite_Table_Alt_Right
	jr Find_Enemy_Sprite_End
Find_Enemy_Sprite_Right:
	ld hl,Enemy_Sprite_Table_Right
	jr Find_Enemy_Sprite_End


Draw_Gray_Screen:                       ;Draws grayscale image w/layers at hl, de
    push de                             ;Save pointers
    ld de,$fc00                         ;Copy to bit plane 1
    ld bc,1024
    ldir
    pop hl                              ;Then restore the other pointer
    ld de,$ca00                         ;and copy to bit plane 2
    ld bc,1024
    ldir
    ret


Copy_To_Buffers:						;Copy grayscale image w/layers at hl, de to buffers
	push de
	ld de,Gfx_Buffer_2
	ld bc,Pic_Len
	ldir
	pop hl
	ld de,Gfx_Buffer_1
	ld bc,Pic_Len
	ldir
	ret


Clear_Back:								;Clear background buffer
	ld hl,Back_Buffer_1
	ld de,Back_Buffer_1+1
	ld (hl),0
	ld bc,3*Pic_Len
	ldir
	ret


Copy_From_Buffer:						;Copy contents of buffer to screen
	ld hl,Gfx_Buffer_1
	ld de,Gfx_Buffer_2
	jr Draw_Gray_Screen


Restore_Gray_Sprite:					;Erase 32*a grayscale sprite at (e,d) from background buffer
	ld (Copy_Gfx_Block_Height),a
	call FindPixel						;Get address of first layer
	push hl
	push hl
	push hl
	ld bc,Gfx_Buffer_1					;in screen buffer
	add hl,bc
	ex de,hl							;and put that in de
	pop hl
	ld bc,Back_Buffer_1					;Now get the address of the first layer in screen data
	add hl,bc							;Put that in hl
	call Copy_Gfx_Block					;and restore this layer
	pop hl								;then do the same for the other layer
	ld bc,Gfx_Buffer_2
	add hl,bc
	ex de,hl
	pop hl
	ld bc,Back_Buffer_2
	add hl,bc
;	jr Copy_Gfx_Block


Copy_Gfx_Block:							;Copy x lines of 3 bytes each from hl to de
Copy_Gfx_Block_Height	equ	$+1
	ld a,18								;Set number of lines to copy
Copy_Gfx_Block_Loop:
	ldi
	ldi
	ldi
	ldi
	ld bc,Screen_Width-4				;and init next-line-addition
	add hl,bc							;then move both pointers to next line
	ex de,hl
	add hl,bc
	ex de,hl
	dec a
	jr nz,Copy_Gfx_Block_Loop			;Loop until all lines have been copied
	ret


Sub_Delay:
	ld hl,(Sub_Delay_Coeff)
	jr Delay
Title_Delay:
	ld hl,Title_Delay_Coeff
	jr Delay
Title_Key_Delay:
    ld hl,Title_Key_Delay_Coeff
Delay:									;Run delay with length in hl
    push hl
    push hl
    pop de
    pop bc
    ldir
    ret


_vnewline:								;Move cursor to next line
    ld hl,(_penCol)
    ld a,(Start_Of_Line)
    ld l,a
    ld a,6
    add a,h
    ld h,a
    ld (_penCol),hl
    ret


Draw_Sprite:							;Draw sprite w/position in bc, data (width, height, dark, light, mask) at hl
    ld d,(hl)
    inc hl
    ld a,(hl)
	ld (Draw_Sprite_Height),a
	ld (VeryFastGraySprite8_height),a
	ld (VeryFastGraySprite8_height1),a 
	add a,a
	ld (VeryFastGraySprite8_height2),a
	inc hl
	ld e,b
	ld b,c
	ld c,e
Draw_Sprite_Loop:
    push de
    push bc
	push hl
    call VeryFastGraySprite8
	pop hl
Draw_Sprite_Height	equ	$+1
	ld a,0
	ld c,a
	add a,c
	add a,c
	ld c,a
	ld b,0
	add hl,bc
    pop bc
    pop de
    ld a,8
    add a,b
    ld b,a
    dec d
    jr nz,Draw_Sprite_Loop
    ret


;===============Very Fast Gray Sprite Routine (8x8) by Donald Straney==============
;
;48 bytes/4609 cycles (worst case)
;Inputs: hl-Pointer to sprite, dark layer first, then light layer,
;		 then mask
;		 bc=x,y position to draw sprite
;
;Outputs: sprite drawn to screen, b=0, c=15
;Destroyed: af,bc,de,hl,ix
;

layer1	equ	Gfx_Buffer_1
layer2	equ	Gfx_Buffer_2

VeryFastGraySprite8:
	push hl					;save sprite address
VeryFastGraySprite8_FP:
	ld l,c					;and find location on screen
	ld h,0					;hl=y
	add hl,hl				;*2
	add hl,hl				;*4
	add hl,hl				;*8
	add hl,hl				;*16
	ld a,b					;a=x
	rra						;/2
	rra						;/4
	rra						;/8
	and %00011111
	add a,l					;add that to offset
	ld l,a					;hl=address (offset)
	ld a,b
	and %00000111			;a=shift amount
VeryFastGraySprite8_params:
	ld b,a					;find amount to patch shift code
	ld a,7
	sub b					;a=(7-shift)
	add a,a
	ld c,a
	add a,a
	add a,c					;a=(7-shift)*6
	ld (VeryFastGraySprite8_shiftr),a	;jr [right=(7-shift)*6]
	ld a,b
	add a,a
	ld c,a
	add a,a
	add a,c					;a=shift*6
	ld (VeryFastGraySprite8_shiftl),a	;jr [left=shift*6]
	pop ix					;restore sprite address
	push hl					;and find addresses of layers
	ld de,layer2
	add hl,de
	ex de,hl				;de=layer2
	pop hl
	ld bc,layer1
	add hl,bc				;hl=layer1
VeryFastGraySprite8_height	equ	$+1
	ld b,8					;init row counter
VeryFastGraySprite8_loop:
	push bc					;save counter
	ld b,(ix+0)				;and get first layer in b
VeryFastGraySprite8_height1	equ	$+2
	ld c,(ix+8)				;second layer in c
VeryFastGraySprite8_height2	equ	$+2
	ld a,(ix+16)			;mask in a
	push af					;(save them for next byte)
	push bc
VeryFastGraySprite8_shiftr	equ	$+1
	jr $
	srl a					;and shift all of them!
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	cpl
	push af
	and (hl)				;and draw to layer 1
	or b
	ld (hl),a
	pop af
	ex de,hl				;then layer 2
	and (hl)
	or c
	ld (hl),a
	inc hl					;move to next byte
	inc de
	pop bc					;and restore the gfx for drawing the next byte
	pop af
VeryFastGraySprite8_shiftl	equ	$+1
	jr $
	sla a					;shift all of them left this time
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	cpl
	push af
	and (hl)				;again, draw to layer 2
	or c
	ld (hl),a
	pop af
	ex de,hl				;and layer 1
	and (hl)
	or b
	ld (hl),a
	ld bc,15				;now move to next line
	add hl,bc
	ex de,hl
	add hl,bc
	ex de,hl
	inc ix					;point to the next byte in sprite
	pop bc					;restore loop counter
	dec b
	jp nz,VeryFastGraySprite8_loop	;and loop!
	ret


;Taken from Diederik Kingma's 83+ grayscale package
;================================================
; byteGetMirror
;================================================
byteGetMirror:
	ld e,a
	and %00001111
	ld b,0 \ ld c,a
	ld hl,LUT1
	add hl,bc
	ld a,(hl) ;first nibble ready
	ld d,a
	ld a,e
	rra
	rra
	rra
	rra
	and %00001111
	ld c,a
	ld hl,LUT2
	add hl,bc
	ld a,(hl) ;second nibble ready
	or d      ;combine nibbles
	ret

LUT1:
	.db %00000000, %10000000, %01000000, %11000000 ;1st nibble
	.db %00100000, %10100000, %01100000, %11100000
	.db %00010000, %10010000, %01010000, %11010000
	.db %00110000, %10110000, %01110000, %11110000
LUT2:
	.db %00000000, %00001000, %00000100, %00001100 ;2nd nibble
	.db %00000010, %00001010, %00000110, %00001110
	.db %00000001, %00001001, %00000101, %00001101
	.db %00000011, %00001011, %00000111, %00001111


;============================================================ 
; DrawMap -- draws an 8x16 tilemap  [Assembly Coder's Zenith] 
;  input: HL = pointer to tilemap to draw 
;  total: 34b/91529t (including ret and entire GridPutSprite) 
;============================================================ 
DrawMap:
  ld de,$0000            ; start drawing at (0, 0) 
DrawMapL:
  push hl                ; save map pointer  
  ld l,(hl)              ; get the current tile
  ld h,0                 ; clear the upper byte
  add hl,hl              ; multiply by 8 (* 2)
  add hl,hl              ; * 4
  add hl,hl              ; * 8

;Modifications
  ld b,h
  ld c,l
  add hl,bc		;* 16
  add hl,bc		;* 24

  ld bc,Tileset_Addr          ; load the start of the sprites
  add hl,bc              ; add togehter to get correct sprite
  push de                ; save sprite draw location
  call GridPutSprite     ; draw the sprite
  pop de                 ; restore sprite draw location
  pop hl                 ; restore map pointer
  inc hl                 ; go to next tile
  inc e                  ; move location to the right
  bit 4,e                ; if the 5th bit (e = 16), we're done
  jr z,DrawMapL          ; only jump if the row isn't complete
  ld e,0                 ; draw at the left again
  inc d                  ; move location down a row
  bit 3,d                ; check if it's done (e = 8)
  ret nz                 ; return if we're done
  jr DrawMapL            ; jump back to the top of the loop  


;=========================================================== 
; GridPutSprite:                       [routine by Dan Eble] 
;  Puts an aligned sprite at (E, D), where HL -> Sprite 
;=========================================================== 
; IN : D  = y (0-7 inclusive) 
;      E  = x (0-15 inclusive) 
;      HL-> sprite 
; 
; OUT: AF, BC, DE, HL, IX modified 
; Current total : 28b/567t (not counting ret) 
;=========================================================== 

GridPutSprite:
  push hl
  pop ix					; ix-> sprite
  srl d          		; de = 128y+x (16 bytes/row, 8 rows)
  rra
  and $80
  or e           		; add x offset (remember x <= 15)
  ld e,a
  ld hl,Back_Buffer_1            ; hl-> video ram
  add hl,de              ; hl-> video ram + offset
  ld b,8                 ; initialize loop counter
  ld de,$10              ; initialize line increment
GPS_Loop:
;  ld a,(ix+0)    		; get byte from sprite

;Modifications
;  or (ix+8)
  ld a,(ix+16)
  cpl
  push af
  and (hl)
  or (ix+0)
  ld (hl),a
  ld c,h
  ld a,h
  add a,$04
  ld h,a
  pop af
  and (hl)
  or (ix+8)
  ld (hl),a
  ld a,h
  add a,$04
  ld h,a
  ld a,(ix+16)
  or (hl)
Draw_Mask_Patch	equ	$
  ld (hl),a		;$77=ld (hl),a		$00=nop
  ld h,c

;  ld (hl),a      		; put byte on screen
  inc ix         		; move to next byte in sprite
  add hl,de     			; move to next line on screen
  djnz GPS_Loop
  ret


;
; Lite86 II Decompression Routine
; written by Kirk Meyer
;
; NOTE: This must be called as a function
;       unless you modify the "ret z" line.
;
; Inputs:
;   HL -> the compressed data
;   DE -> where to store the output
;
; Outputs:
;   AF, BC, DE, HL trashed

Decompress:
 ld a,(hl)
 inc hl
 scf
 adc a,a
 .db $06
DecompressLoop:
 add a,a
 jr z,Decompress
 jr c,DecompressChunk
 ldi
 jr DecompressLoop
DecompressChunk:
 ld b,a
 ld c,$1F
 ld a,(hl)
 cp c
 ret z
 push bc 
 inc hl
 ld b,(hl)
 inc hl
 push hl
 ld l,b
 ld b,a
 or c
 rlca
 rlca
 rlca
 ld h,a
 ld a,b
 and c
 add a,3
 ld c,a
 ld b,0
 add hl,de
 ldir
 pop hl
 pop af
 jr DecompressLoop


;---------------------------------------------------------------------
; The Eble-Yopp-Yopp-Eble-Eble-Eble-Yopp-Eble Faster FindPixel Routine
; 32-39 bytes/109 t-states not counting ret or possible push/pop of BC
;---------------------------------------------------------------------
; Input:  D = y
;         E = x
; Output: HL= address of byte in video memory
;         A = bitmask (bit corresponding to pixel is set)
;         C is modified
;         Flags are modified
;
; +-----------+
; |(0,0)      |  <- Screen layout
; |           |
; |   (127,63)|  Note: make sure coordinates are within bounds
; +-----------+
;
;---------------------------------------------------------------------
FindPixel:
        ld hl,FP_Bits   ; HL -> bitmask table
        ld a,e          ; A = x coordinate
        and $07         ; A = bit offset
        or l          ; add bit offset to HL (the addition to L will
        ld l,a          ;  not carry, because the table is aligned)
        ld c,(hl)       ; C = bitmask (saved for later)
;36 t-states up to this point

;MODIFICATION
;        ld h,$3F        ; to understand this, draw a picture
		ld h,0

        ld a,d          ; A = y coordinate (top two bits clear)
        add a,a         ; does not carry
        add a,a         ; does not carry
        ld l,a
        ld a,e
        rra             ; puts 0 into the high bit
        add hl,hl       ; does not carry
        rra             ; puts 0 into the high bit
        add hl,hl       ; does not carry
        rra             ; puts 0 into the high bit
        or l
        ld l,a
        ld a,c          ; A = bitmask (saved from above)
;109 t-states up to this point
        ret

.pad (($+8) & 0FFF8h)   ; align FP_Bits on the next 8-byte boundary
FP_Bits: .db $80,$40,$20,$10,$08,$04,$02,$01



;====================================================================
; DisplayHL:  (idea from SCaBBy/Mardell)    [Assembly Coder's Zenith]
;  Display HL as a 5-digit decimal number with no leading zeros
;  out: AF, HL, BC, DE destroyed
;====================================================================
DisplayHL:
 ld c,'0'  				; save ascii value for zero
 ld de,_OP1+5			; point to end of the buffer
 xor a				; zero terminate string
 ld (de),a				; set last byte to zero
DisplayHLl:
 call _divHLby10		; next digit
 dec de				; next position
 add a,c				; convert to ascii
 ld (de),a				; save digit
 ld a,h				; load upper byte
 or l					; check with lower byte for zero
 jr nz,DisplayHLl			; loop
 ex de,hl				; point to buffer
; jp _vputs			; print number, we're done
 ld a,Gfx_Buffer_1/256
 ld (fvputc_vidplane),a
 jp fvputs


;Thanks to Matt Johnson for this code!
Open_Gray:
 ld hl,Vector_Table_Addr	         ; Source = $8E00 (Start of vector table)
 ld de,Vector_Table_Addr+1		     ; Destination = $8E01
 ld (hl),ISR_Addr/256                ; Set first byte so it is copied to all of them ($8F8F)
 ld bc,257                           ; 256 bytes (size of vector table)
 ldir                                ; (DE) <- (HL), BC=BC-1, Loop till B=0
 ld hl,int_copy                      ; Source = grayscale interrupt routine)
 ld de,ISR_Addr                      ; Destination = $8F8F (as specified in vector table)
 ld bc,int_end-int_start             ; Length of custom interrupt routine
 ldir                                ; (DE) <- (HL), BC=BC-1, Loop till B=0
 ld a,Vector_Table_Addr/256          ; Set up address of vector table
 ld i,a                              ;  MSB = $8E
                                     ;  LSB = (byte supplied by data bus - random)
 im 2                                ; Interrupt Mode 2
 ei                                  ; Enable Interrupts
 ret                                 ; Return to calling program


;From grayscale routines by Dux Gregis
Close_Gray:
    im 1
    ld a,$3C                ;make sure video mem is active
    out (0),a
clrMem:             ;clear video mem and graph mem
    call _clrLCD
clrGrf:             ;clear graph mem
    ld hl,_plotSScreen+6
    ld de,_plotSScreen+7
    ld (hl), 0
    ld bc,1024
    ldir
    ret

.end