;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Operation Neptune Clone                                        /
;/                              Written by Donald Straney 9/16/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;Status display code/data

Start_Show_Status:
	ld a,1									;Set display flag
	ld (Draw_Status),a
	ld a,Status_Delay						;and init delay
	ld (Status_Disp_Counter),a
	ret

Draw_Status_Disp:
	sub 2
	jr z,Draw_Status_Disp_Erase				;(If we're supposed to erase the status display, then do that)
	ld hl,Status_Spr						;First draw the status background sprite
	ld bc,Status_Pos
	call Draw_Sprite
	di
	ld a,(Num_Ink_Pellets)					;then display the number of remaining ink pellets,
	ld l,a
	ld h,0
	ld de,Status_Pos+8+(1*256)
	ld (_penCol),de
	call DisplayHL
	ld a,(Oxygen)							;and finally, the amount of oxygen left
	ld l,a
	ld h,0
	ld de,Status_Pos+25+(1*256)
	ld (_penCol),de
	call DisplayHL
	ei
	ret
Draw_Status_Disp_Erase:
	xor a									;First reset the state of the display,
	ld (Draw_Status),a
	ld de,Status_Pos						;then do the actual erasing
	ld a,8
	call Restore_Gray_Sprite
	ld de,Status_Pos+24						;(the status display has to be erased in 2 parts,
	ld a,8									;because of its width)
	jp Restore_Gray_Sprite

Draw_Status_Delay:
	ld a,(Draw_Status)						;If status display is on,
	or a
	ret z
	ld hl,Status_Disp_Counter				;then decrement counter
	dec (hl)
	ret nz
	ld a,2									;If it's hit 0, turn off display
	ld (Draw_Status),a
	ret

Dec_Lives:
	ld hl,Oxygen							;Take away 1 unit of oxygen
	dec (hl)
	jr z,Dec_Lives_Die						;If it's empty...
	ld a,(hl)
	and %11111100							;If not, see if a tank has been used up (oxygen|4=0)
	cp (hl)
	jr nz,Dec_Lives_NotEnd					;If there's still some left in the tank, show the remaining amount in the status display, and make the sub flash
	ld sp,(End_Game_SP)						;Otherwise, go back to the first screen
	call Start_Sub_Flash
	jp Start_Game
Dec_Lives_Die:
	ld sp,(End_Game_SP)
	jp End_Game
Dec_Lives_NotEnd:
	call Start_Show_Status
;	jr Start_Sub_Flash


Start_Sub_Flash:
	ld a,Sub_Flash_Num						;First init delays,
	ld (Sub_Flash_Num_Counter),a
	ld a,Sub_Flash_Time
	ld (Sub_Flash_Counter),a
	ld a,1									;then enable sub flashing
	ld (Sub_Flash),a
	ret


Sub_Flash_Delay:
	ld a,(Sub_Flash)						;Is the sub flashing?
	or a
	ret z									;If it isn't, don't bother with the rest of this stuff
	ld hl,Sub_Flash_Counter					;Otherwise, decrement the flash timer
	dec (hl)
	ret nz
	ld (hl),Sub_Flash_Time					;If it timed out, reset it
	ld hl,Sub_Flash_Num_Counter				;and decrement the number-of-flashes counter
	dec (hl)
	jr z,Sub_Flash_Delay_End				;(if it's at 0, stop the flashing!!! before it's too late!!!)
	ld hl,(Sub_Sprite)						;Now check the sub sprite
	ld de,sub_blank							;Is the sub "off"?
	call _cphlde
	jr z,Sub_Flash_Delay_TurnOn				;If it is, turn it "on"
Sub_Flash_Delay_TurnOff:					;Otherwise, switch it "off"
	ld (Sub_Sprite_Save),hl					;First save the sprite used for the sub
	ld (Sub_Sprite),de						;then replace it with a blank sprite
	ret
Sub_Flash_Delay_TurnOn:
	ld hl,(Sub_Sprite_Save)					;When turning it on, restore the sub sprite
	ld (Sub_Sprite),hl						;from where it was saved
	ret										;That's all for now, folks!
Sub_Flash_Delay_End:
	xor a									;Turn off the sub flash,
	ld (Sub_Flash),a
	ld hl,(Sub_Sprite_Save)
	ld (Sub_Sprite),hl						;and make sure that it stays on, not off!
	ret


Status_Spr:
 .db 5,8
 .db $00,$81,$99,$BD,$BD,$99,$81,$FF
 .db $FF,$81,$99,$BD,$BD,$99,$81,$00
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $00,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$00
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $00,$00,$00,$1E,$67,$5E,$00,$FF
 .db $FF,$00,$0C,$40,$3C,$5E,$00,$00
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
; .db $00,$FE,$FE,$FE,$FE,$FE,$FE,$FF
; .db $FF,$FE,$FE,$FE,$FE,$FE,$FE,$00
; .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

 .db $00,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$00
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

 .db $00,$E0,$E0,$E0,$E0,$E0,$E0,$F0
 .db $F0,$E0,$E0,$E0,$E0,$E0,$E0,$00
 .db $f0,$f0,$f0,$f0,$f0,$f0,$f0,$f0

.end